/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>
#include <string.h>
#include <unistd.h>
#include <worldvu/ext.h>

static  PORT(SetupPort);

const   DEFWINDOW(SetupDesktop)
    WIN_CLASS(SetupPort, (STRING)"Setup Module")
    WIN_ATTR(-1, -1, HASMENUBAR, 1, NULL)
ENDDEF

#define MSG_BOOKMARK    msgs[replacement_language * 3]
#define MSG_ADDED   msgs[replacement_language * 3 + 1]
#define MSG_LOADING msgs[replacement_language * 3 + 2]

CONST   char    *msgs[] = {
    "*** Cannot bookmark ***",
    "Bookmark added",
    "Loading...",

#ifdef  __ISO8
    "*** Cannot bookmark ***",
    "Bookmark added",
    "Loading...",

    "*** Nu se poate adauga la preferinte ***",
    "Adaugat la preferinte",
    "Incarcare...",

    "*** Nemohu vytvorit zalozku ***",
    "Zalozka zarazena",
    "Startuji...",

#endif
    NULL
};

static  DEFPORT(SetupPort)
{
    char    path[32];
    FILE    *fp;
    int     bsize;
    int     rtn, row, col;
    char    *env;
    char    buf[200];
    char    fld[80];
    char    *p;

    switch(event) {
    case VU_KEYBOARD:
        if(vu->vuimage) {
            if(p1 == KEY_EXIT || p1 == KEY_SYSREQ || p1 == 3) {
                ++shutdown_flag;
                return TRUE;
            }
        }
        if(p1 == 11 || p1 == KEY_F2 || p1 == (ALTKEY | 'K')) {
            env = getenv("SETUP_BOOKMARK");
            if(!env)
                break;

            strcpy(fld, env);
            while(NULL != (p = strchr(fld, '_')))
                *p = ' ';
            sprintf(buf, "\"setup -> %s\" vu setup %s\n", fld, env);
            if(add_bookmark(buf)) {
                front->vustatline = (STRING)MSG_BOOKMARK;
                front->vustatcolor = STAT_ERROR;
                VUTermAlert(1);
                fixcursor();
            }
            else {
                front->vustatline = (STRING)MSG_ADDED;
                front->vustatcolor = STAT_GOOD;
                fixcursor();
            }
            return TRUE;
        }
        break;
    case VU_CREATE:
        strcpy(path, "/tmp/.VU");
        strcat(path, strint(getppid(), 0));
        fp = fopen(path, "r");
        bsize = ((rows - 2) * cols * sizeof(CELL));
        if(fp) {
            vu->vuimage = (char *)malloc(bsize);
            vu->vustatline = (STRING)MSG_LOADING;
            vu->vustatcolor = STAT_BUSY;
            fixcursor();
            fread(imageport, bsize + (cols * sizeof(CELL)), 1, fp);
            fclose(fp);
            remove(path);
            memcpy(vu->vuimage, &imageport[cols], bsize);
            VUDispatch();
            vu->vustatline = (STRING)"";
            vu->vustatcolor = STAT_INFO;
        }
        break;
    case VU_PAINT:
        p2 = deskpattern;
        if(!vu->vuimage)
            break;

        rtn = USEPORT(VUDeskPort);
        if(rtn && vu->vuimage) {
            p2 = TRANSPARENT;
            for(row = 1; row < (rows - 2); ++row)
                for(col = 0; col < cols; ++col)
                    VUDrawCell(
                        ((CELL *)(vu->vuimage))[row * cols + col],
                        row + 1, col);
        }
        return rtn;
    default:
        break;
    }
    return USEPORT(VUUserPort);
}
