/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *  User desktop for QNX.
 */

#include <vu-config.h>

#ifdef  HAVE_PWD_H
#include <pwd.h>
#include <grp.h>

#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <pwd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/socket.h>

#define REAL_ID pwd->pw_gecos
#define PROC_FS "/proc/%d/environ"

#ifndef REAL_ID
#define REAL_ID pwd->pw_comment
#endif

static VU finger(struct passwd *pwd);

static  PORT(NoticePort);
static  PORT(SendPort);
static  PORT(WhoPort);
static  PORT(FingerPort);

static  BOOL    blocked = FALSE;
static  char    to[17];
static  char    text[13 * 77];
static  char    my_userid[17];

#define MSG_INVALID     (char *)msgs[replacement_language * 8]
#define MSG_ILLEGAL     (char *)msgs[replacement_language * 8 + 1]
#define MSG_OFFLINE     (char *)msgs[replacement_language * 8 + 2]
#define MSG_BUSY        (char *)msgs[replacement_language * 8 + 3]
#define MSG_FLASH       (char *)msgs[replacement_language * 8 + 4]
#define MSG_NOTICE      (char *)msgs[replacement_language * 8 + 5]
#define MSG_LOADING     (char *)msgs[replacement_language * 8 + 6]
#define MSG_IFLASH      (char *)msgs[replacement_language * 8 + 7]

CONST   char    *msgs[] = {
    "*** Invalid userid ***",
    "*** Illegal or unknown userid ***",
    "*** Not currently online ***",
    "*** User busy; try again later ***",
    "Flash message from %s <%s>",
    "Flash Notice",
    "Loading...",
    "Flash message from %s",

#ifdef  BUILD_LOCALIZED
    "*** Utente invalido ***",
    "*** Utente illegale o sconosciuto ***",
    "*** Utente non collegato ***",
    "*** Utente occupato; riprova tra qualche minuto ***",
    "Messaggio da %s <%s>",
    "Messaggio",
    "Caricamento in corso...",
    "Messaggio da %s",


    "*** Cont incorect ***",
    "*** Cont invalid sau necunoscut sistemului ***",
    "*** Nu este intrat in sistem ***",
    "*** In acest moment utlizatorul este ocupat; incercati din nou ***",
    "Mesaj urgent de la %s <%s>",
    "Mesaj urgent",
    "Incarcare...",
    "Mesaj urgent de la %s",

    "*** Neplatny ucet ***",
    "*** Neplatny nebo neznamy ucet ***",
    "*** Uzivatel neni momentalne pripojen ***",
    "*** Uzivatel je zaneprazdnen; zkuste pozdeji ***",
    "Hlaska od %s <%s>",
    "Hlaska",
    "Startuji...",
    "Hlaska od %s",

#endif
    NULL
};

CONST   DEFWINDOW(Notice)
    WIN_CLASS(NoticePort, NULL)
    RES_ATTR(14, 66, HASSHADOW | HASBORDER | HASMENUBAR, 40, WIN_FLASH,
        (STRING)"Press <CONTROL-D> to continue")

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, (STRING)"Premi <CONTROL-D> per continuare")
    REPLACE_ROMANIAN(NULL, (STRING)"Apasati <CONTROL-D> pentru continuare")
    REPLACE_CZECH(NULL, (STRING)"Chcete-li pokracovat, stisknete <CONTROL-D>")
#endif
ENDDEF


CONST   DEFWINDOW(Finger)
    WIN_CLASS(NoticePort, NULL)
    RES_ATTR(14, 76, HASSHADOW | HASBORDER | HASMENUBAR, 40, WIN_FLASH,
        (STRING)"Press <CONTROL-D> to continue")

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, (STRING)"Premi <CONTROL-D> per continuare")
    REPLACE_ROMANIAN(NULL, (STRING)"Apasati <CONTROL-D> pentru continuare")
    REPLACE_CZECH(NULL, (STRING)"Chcete-li pokracovat, stisknete <CONTROL-D>")
#endif
ENDDEF


CONST   DEFWINDOW(GetFinger)
    WIN_CLASS(FingerPort, (STRING)"Finger by userid")
    RES_ATTR(7, 28, HASSHADOW | HASBORDER | HASMENUBAR, 40, WIN_FINGER,
        (STRING)"Press <ESCAPE> to continue")

    EDITID_CONTROL((STRING)to,      2,10, 16, VU_FINGER_ID, NULL)

    REPLACE_ENGLISH((STRING)"Finger by userid", (STRING)"Press <ESCAPE> to continue")
    TEXT_CONTROL((STRING)"~Userid:", 2, 2, 1, 8, VU_FINGER_ID)
    BUTTON_CONTROL((STRING)" ~Accept ", 4, 4, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)" ~Cancel ", 4,16, 8, VU_CANCEL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Informazioni utente", (STRING)"Premi <ESC> per continuare")
    TEXT_CONTROL((STRING)"~Utente:", 2, 2, 1, 8, VU_FINGER_ID)
    BUTTON_CONTROL((STRING)"~Conferma", 4, 4, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)"~Annulla ", 4,16, 8, VU_CANCEL)

    REPLACE_ROMANIAN((STRING)"Detalii despre un cont", (STRING)"Apasati <ESCAPE> pentru continuare")
    TEXT_CONTROL((STRING)"~Cont:",   2, 2, 1, 8, VU_FINGER_ID)
    BUTTON_CONTROL((STRING)" ~Corect ", 4, 4, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)"~Abandon ", 4,16, 8, VU_CANCEL)

    REPLACE_CZECH((STRING)"Finger podle uctu", (STRING)"Chcete-li pokracovat, stisknete <ESC>")
    TEXT_CONTROL((STRING)"~Ucet:", 2, 2, 1, 8, VU_FINGER_ID)
    BUTTON_CONTROL((STRING)"   ~OK   ", 4, 4, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)" ~Zrusit ", 4,16, 8, VU_CANCEL)

#endif

ENDDEF

#ifdef  PROC_FS
#define WHO_WIDTH 64
#define LINE_SEG "2u17r2d2u9r2d2u6r2d2u9r2d2u9r2d"
#define ROM_LINE_SEG "2u17r2d2u8r2d2u7r2d2u9r2d2u9r2d"
#else
#define WHO_WIDTH 46
#define LINE_SEG "2u17r2d2u9r2d2u6r2d"
#define ROM_LINE_SEG "2u17r2d2u8r2d2u7r2d"
#endif

CONST   DEFWINDOW(Who)
    WIN_CLASS(WhoPort, (STRING)"Who is online")
    RES_ATTR(12, WHO_WIDTH, HASSHADOW | HASBORDER | HASMENUBAR, 40, WIN_WHO,
        (STRING)"Press <ESCAPE> to exit")

    REPLACE_ENGLISH((STRING)"Who is online", (STRING)"Press <ESCAPE> to exit")
    BUTTON_CONTROL((STRING)" M~essage", 4, WHO_WIDTH - 11, 8, VU_MESSAGE)
    BUTTON_CONTROL((STRING)" ~Finger ", 6, WHO_WIDTH - 11, 8, VU_FINGER)
    BUTTON_CONTROL((STRING)" ~Cancel ", 8, WHO_WIDTH - 11, 8, VU_CANCEL)
    BOX_CONTROL((STRING)"", 3, 1, 8, WHO_WIDTH - 13, VU_SELECT)
    FILL_CONTROL(1, 1, 2, WHO_WIDTH - 13, 1)
    LINE_CONTROL((STRING)LINE_SEG, 3, 1)
    TEXT_FIELD((STRING)"Userid", 2, 2, 1, 6, 2)
    TEXT_FIELD((STRING)"Port",   2,19, 1, 4, 2)
    TEXT_FIELD((STRING)"Login",  2,28, 1, 5, 2)
#ifdef  PROC_FS
    TEXT_FIELD((STRING)"Language", 2, 34, 1, 8, 2)
    TEXT_FIELD((STRING)"Module",   2, 43, 1, 8, 2)
#endif

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Utenti collegati", (STRING)"Premi <ESC> per uscire")
    BUTTON_CONTROL((STRING)" M~essaggio", 4, WHO_WIDTH - 12, 10, VU_MESSAGE)
    BUTTON_CONTROL((STRING)" ~Finger ", 6, WHO_WIDTH - 11, 8, VU_FINGER)
    BUTTON_CONTROL((STRING)"~Annulla ", 8, WHO_WIDTH - 11, 8, VU_CANCEL)
    BOX_CONTROL((STRING)"", 3, 1, 8, WHO_WIDTH - 13, VU_SELECT)
    FILL_CONTROL(1, 1, 2, WHO_WIDTH - 13, 1)
    LINE_CONTROL((STRING)LINE_SEG, 3, 1)
    TEXT_FIELD((STRING)"Utente", 2, 2, 1, 6, 2)
    TEXT_FIELD((STRING)"Porta",  2,19, 1, 5, 2)
    TEXT_FIELD((STRING)"Login",  2,28, 1, 5, 2)
#ifdef  PROC_FS
    TEXT_FIELD((STRING)"Language", 2, 34, 1, 8, 2)
    TEXT_FIELD((STRING)"Module",   2, 43, 1, 8, 2)
#endif

    REPLACE_ROMANIAN((STRING)"Cine este intrat", (STRING)"Apasati <ESCAPE> pentru terminare")
    BUTTON_CONTROL((STRING)" M~esaj  ", 4, WHO_WIDTH - 11, 8, VU_MESSAGE)
    BUTTON_CONTROL((STRING)"Deta~lii ", 6, WHO_WIDTH - 11, 8, VU_FINGER)
    BUTTON_CONTROL((STRING)"~Abandon ", 8, WHO_WIDTH - 11, 8, VU_CANCEL)
    BOX_CONTROL((STRING)"", 3, 1, 8, WHO_WIDTH - 13, VU_SELECT)
    FILL_CONTROL(1, 1, 2, WHO_WIDTH - 13, 1)
    LINE_CONTROL((STRING)ROM_LINE_SEG, 3, 1)
    TEXT_FIELD((STRING)"Cont",   2, 2, 1, 6, 2)
    TEXT_FIELD((STRING)"Linie",  2,19, 1, 5, 2)
    TEXT_FIELD((STRING)"Intrat", 2,27, 1, 6, 2)
#ifdef  PROC_FS
    TEXT_FIELD((STRING)"Language", 2, 34, 1, 8, 2)
    TEXT_FIELD((STRING)"Module",   2, 43, 1, 8, 2)
#endif

    REPLACE_CZECH((STRING)"Kod je pripojen", (STRING)"Pro ukonceni stisknete <ESC>")
    BUTTON_CONTROL((STRING)" H~laska ", 4, WHO_WIDTH - 11, 8, VU_MESSAGE)
    BUTTON_CONTROL((STRING)" ~Finger ", 6, WHO_WIDTH - 11, 8, VU_FINGER)
    BUTTON_CONTROL((STRING)" ~Zrusit ", 8, WHO_WIDTH - 11, 8, VU_CANCEL)
    BOX_CONTROL((STRING)"", 3, 1, 8, WHO_WIDTH - 13, VU_SELECT)
    FILL_CONTROL(1, 1, 2, WHO_WIDTH - 13, 1)
    LINE_CONTROL((STRING)LINE_SEG, 3, 1)
    TEXT_FIELD((STRING)"Ucet", 2, 2, 1, 6, 2)
    TEXT_FIELD((STRING)"Port",   2,19, 1, 4, 2)
    TEXT_FIELD((STRING)"Login",  2,28, 1, 5, 2)
#ifdef  PROC_FS
    TEXT_FIELD((STRING)"Jazyk", 2, 34, 1, 8, 2)
    TEXT_FIELD((STRING)"Akce",   2, 43, 1, 8, 2)
#endif

#endif
ENDDEF

CONST   DEFWINDOW(SendNotice)
    WIN_CLASS(SendPort, (STRING)"Send Flash Message")
    RES_ATTR(19, 66, EDITBAR | HASSHADOW | HASBORDER | HASMENUBAR | WORDWRAP, 24, WIN_SEND,
        (STRING)"Press <ESCAPE> to exit")
    EDIT_CONTROL((STRING)text,          3, 1, 12, 64, VU_SEND_TXT, NULL)
    TEXT_FIELD((STRING)to,              1,10, 1, 32, 2)

    REPLACE_ENGLISH((STRING)"Send Flash Message", (STRING)"Press <ESCAPE> to exit")
    TEXT_CONTROL((STRING)"Send to:",        1, 1, 1, 8, VU_SEND_TO)
    BUTTON_CONTROL((STRING)" ~Accept ",16, 8, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)" ~Cancel ",16,50, 8, VU_CANCEL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Invio messaggio in diretta", (STRING)"Premi <ESC> per uscire")
    TEXT_CONTROL((STRING)"Invia a:",        1, 1, 1, 8, VU_SEND_TO)
    BUTTON_CONTROL((STRING)"~Conferma",    16, 8, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)"~Annulla ",    16,50, 8, VU_CANCEL)

    REPLACE_ROMANIAN((STRING)"Trimiteti mesaj urgent", (STRING)"Apasati <ESCAPE> pentru terminare")
    TEXT_CONTROL((STRING)"Catre contul:",   1, 1, 1, 8, VU_SEND_TO)
    BUTTON_CONTROL((STRING)" ~Corect ",    16, 8, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)"~Abandon ",    16,50, 8, VU_CANCEL)

    REPLACE_CZECH((STRING)"Odeslani hlasky", (STRING)"Pro ukonceni stisknete <ESC>")
    TEXT_CONTROL((STRING)"  Pro: ",  1, 1, 1, 7, VU_SEND_TO)
    BUTTON_CONTROL((STRING)"   ~OK   ",16, 8, 8, VU_ACCEPT)
    BUTTON_CONTROL((STRING)" ~Zrusit ",16,50, 8, VU_CANCEL)

#endif
ENDDEF

static  VU      netfinger(char *id)
{
    static  char    *argv[3] = {"/usr/tycho/sys/netfinger", to, NULL};
    int     pp[2];
    int     pid;
    char    *tp;
    int     len, status;
    char    *statln;
    int statcolor;

    statln = (char *)front->vustatline;
    statcolor = front->vustatcolor;

    front->vustatline = (STRING)MSG_LOADING;
    front->vustatcolor = STAT_BUSY;
    fixcursor();

    strcpy(to, id);
    pipe(pp);
    pid = fork();
    if(pid == -1)
        return NULL;

    if(!pid) {
        close(1);
        dup2(pp[1], 1);
        close(2);
        open("/dev/null", O_WRONLY);
        execvp(*argv, argv);
        exit(-1);
    }
    close(pp[1]);
    len = read(pp[0], text, 76 * 13);
    if(len < 0)
        len = 0;
    text[len] = 0;

    close(pp[0]);
    waitpid(pid, &status, 0);
    if((status & 1) || !len)
        return NULL;

    tp = strchr(text, '\n');
    *(tp++) = 0;
    front->vustatline = (STRING)statln;
    front->vustatcolor = statcolor;

    return VUCreateQ(Finger, (STRING)text, -1, -1, tp);
}

static  VU      finger(struct passwd *pwd)
{
    char    path[128];
    int             len;
    int             fd;

    if(!pwd)
        return NULL;

    strcpy(text, "no plan\n");
    strcpy(path, pwd->pw_dir);
    len = strlen(path);
    if(path[len - 1] != '/')
        strcat(path, "/");
    strcat(path, ".plan");
    fd = open(path, O_RDONLY);
    if(fd > 0) {
        len = read(fd, text, 64 * 12);
        if(len < 0)
            len = 0;
        text[len] = 0;
        close(fd);
    }
    return VUCreateQ(Notice, (STRING)REAL_ID, -1, -1, text);
}

static  DEFPORT(NoticePort)
{
    if(event == VU_KEYBOARD) {
        if(p1 == 27)
            return TRUE;

        if(p1 == 0x04 || p1 == 26)
            p1 = 27;
    }
    return USEPORT(VUMsgboxPort);
}

static  DEFPORT(FingerPort)
{
    struct  passwd  *pwd;
    VU      child;

    switch(event) {
    case VU_CREATE:
        to[0] = 0;
        if(access("/usr/tycho/spool/flash", R_OK))
            blocked = TRUE;

        break;
    case VU_BUTTON:
        if(p1 != VU_ACCEPT)
            break;

        if(!strchr(to, '@')) {
            pwd = getpwnam(to);
            endpwent();
            if(!pwd) {
                vu->vustatline = (STRING)MSG_INVALID;
                vu->vustatcolor = STAT_ERROR;
                VUTermAlert(1);
                fixcursor();
                VUPostEvent(vu, VU_KEYBOARD, 0x15, 0);
                return TRUE;
            }
            child = finger(pwd);
            child->vuparent = vu->vuparent;
            vu->vuparent = child;
        }
        else {
            child = netfinger(to);
            if(!child) {
                vu->vustatline = (STRING)MSG_INVALID;
                vu->vustatcolor = STAT_ERROR;
                VUTermAlert(1);
                fixcursor();
                VUPostEvent(vu, VU_KEYBOARD, 0x15, 0);
                return TRUE;
            }
            child->vuparent = vu->vuparent;
            vu->vuparent = child;
        }
    default:
        break;
    }
    return USEPORT(VUWindowPort);
}

static  DEFPORT(WhoPort)
{
    char    *argv[] = {"/usr/tycho/sys/userlist", NULL};
    int     fd[2];
    char    *p;
    FILE    *fp;
    char    **list;
    char    *item;
    SELECT  opts = (SELECT)vu->vuoptions;
    static  char    buf[18];
    char    lbuf[128];
    VU              child;
    int     pid;
    char    *oldstat;
    int     oldcolor;

    switch(event) {
    case VU_DESTROY:
        list = (char **)vu->vupaint;
        while(*list)
            free(*(list++));
        free(vu->vupaint);
        break;
    case VU_BUTTON:
        list = (char **)vu->vupaint;
        strncpy(buf, list[opts->sel_mark], 16);
        buf[17] = 0;
        strtok(buf, " ");
        child = NULL;

        switch(p1) {
        case VU_MESSAGE:
            child = VUCreateQ(SendNotice, NULL, -1, -1, buf);
            break;
        case VU_FINGER:
            if(!strchr(buf, '@'))
                child = finger(getpwnam(buf));
            else
                child = netfinger(buf);
            endpwent();
            break;
        }
        if(child) {
            child->vuparent = vu->vuparent;
            vu->vuparent = child;
        }
        break;

    case VU_CREATE:
        oldstat = (char *)front->vustatline;
        oldcolor = front->vustatcolor;
        front->vustatline = (STRING)MSG_LOADING;
        front->vustatcolor = STAT_BUSY;
        fixcursor();
        front->vustatline = (STRING)oldstat;
        front->vustatcolor = oldcolor;

        if(!USEPORT(VUSelectPort))
            return FALSE;


        vu->vupaint = (char *)malloc(128 * sizeof(char *));
        list = (char **)vu->vupaint;
        list[0] = 0;
        pipe(fd);
        pid = fork();
        if(pid < 0)
            return FALSE;
        if(pid == 0) {
            dup2(fd[1], 1);
            close(fd[1]);
            close(fd[0]);
            write(1, "USERLIST\n", 9);
            execvp(*argv, argv);
            exit(-1);
        }

        close(fd[1]);
        fp = fdopen(fd[0], "r");
        fgets(lbuf, sizeof(lbuf) - 1, fp);
        while(!feof(fp)) {
            fgets(lbuf, sizeof(lbuf) - 1, fp);
            if(feof(fp))
                break;
            p = strchr(lbuf, '\n');
            *p = 0;
            item = (char *)malloc(strlen(lbuf) + 1);
            strcpy(item, lbuf);
            VUSelectInsert(vu, VUSelectLocate(vu, item, (int(*)(const char *s1, const char *s2)) VUStricmp), item);
        }
        fclose(fp);
        waitpid(pid, NULL, 0);
        opts->sel_mark = opts->sel_base = 0;
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUSelectPort);
}

static  DEFPORT(SendPort)
{
    int             pid;
    int             len;
    static  char    *argv[4] = {"/usr/tycho/sys/postflash", to, text, NULL};

    switch(event) {
    case VU_CREATE:
        text[0] = 0;
        if(vu->vupaint)
            strcpy(to, vu->vupaint);
        else
            to[0] = 0;
        break;
    case VU_BUTTON:
        if(p1 != VU_ACCEPT)
            break;

        if(!text[0])
            strcpy(text, "<eom>\n");

        pid = fork();
        if(pid > 0) {
            sleep(2);
            waitpid(pid, &len, 0);
            pid = len;
        }
        else if(!pid) {
            close(2);
            open("/dev/null", O_WRONLY);
            if(strchr(to, '@'))
                execv("/usr/tycho/sys/netflash", argv);
            else
                execv(*argv, argv);
            exit(-1);
        }

        if(pid) {
            vu->vustatline = (STRING)MSG_BUSY;
            vu->vustatcolor = STAT_ERROR;
            VUTermAlert(1);
            fixcursor();
            /* VUPostEvent(vu, VU_KEYBOARD, 0x13, 0); */
            return TRUE;
        }
    default:
        break;
    }
    return USEPORT(VUWindowPort);
}

DEFPORT(VUUserPort)
{
    char            *argv[2] = {NULL, NULL};
    int                     pid, status;
    DESKOPTS        opts = (DESKOPTS)vu->vuoptions;
    int                     fd;
    int                     len;
    int                     pal = 40;
    static          char            path[128];
    struct          stat    inode;
    struct          passwd  *pwd;
    char            **list = (char **)(vu->vupaint);
    int                     count;
    int                     margin;
    int                     row, col;
    int                     i;
    static          char    *killflash[] = {"/usr/tycho/sys/killflash", NULL};
    char            *env, *name;
    static          char    envmarker[32];
    char            *tp;

    switch(event) {
    case VU_CREATE:
        sprintf(envmarker, "VU_PID=%d", getpid());
        putenv(envmarker);
        name = getlogin();
        if(!name) {
            pwd = getpwuid(getuid());
            endpwent();
            if(pwd)
                name = pwd->pw_name;
            else
                name = "user";
        }
        env = (char *)malloc(32);
        strcpy(env, "LOGNAME=");
        strcpy(env + 8, name);
        putenv(env);
        strcpy(my_userid, name);
        break;

    case VU_PAINT:
        if(!USEPORT(VUDeskPort))
            return FALSE;

        if(!list)
            return TRUE;

        if(shutdown_flag)
            return TRUE;

        count = 0;
        while(list[count])
            ++count;

        if(cols < (count * 10))
            return TRUE;

        margin = (cols - (count * 9)) / (count + 1);
        i = 0;
        row = rows - 2;
        col = margin;
        while(i < count) {
            VUDrawButtonUp((STRING)list[i], row, col, 9, 0);
            col += 9 + margin;
            ++i;
        }
        return TRUE;
    case VU_KEYBOARD:
        if(get_keylabel(p1) == KEY_SYSREQ) {
            VUPostEvent(vu,VU_SHELL,0,0);
            return TRUE;
        }

        if(!opts->desk_notice) {
            switch(get_keylabel(p1)) {
            case KEY_WHO:
                opts->desk_notice = VUCreateQ(Who, NULL, 2, 3, NULL);
                opts->desk_notice->vuparent = vu;
                return TRUE;
            case KEY_FINGER:
                opts->desk_notice = VUCreateQ(GetFinger, NULL, 2, 3, NULL);
                opts->desk_notice->vuparent = vu;
                return TRUE;
            }
        }
        break;
    case VU_SHELL:
        *argv = getenv("SHELL");
        if(!argv[0])
            return TRUE;
        if(!**argv)
            return TRUE;
        if(!strcmp(*argv, "off"))
            return TRUE;
        if(!strcmp(*argv, "noshell"))
            return TRUE;

        VUTermSuspend(TRUE);
        pid = fork();
        if(pid > 0) {
            while(pid != wait(&status))
                ;
        }
        else if(!pid) {
#ifdef  HAVE_SETGROUPS
            setgroups(0, NULL);
#endif
            setuid(getuid());
            setgid(getgid());
            if(geteuid() != getuid())
                exit(-1);
            if(getegid() != getgid())
                exit(-1);
            execv(*argv, argv);
            exit(-1);
        }
        VUTermContinue();
        return TRUE;
    case VU_NOTIFY:
        if(opts->desk_notice) {
            if(opts->desk_ntext) {
                free(opts->desk_ntext);
                opts->desk_ntext = NULL;
            }
            opts->desk_notice = NULL;
        }
        break;
    case VU_IDLE:
        if(opts->desk_locked || opts->desk_notice)
            break;

        if(blocked)
            break;

        if(!opts->desk_shutdown && !shutdown_flag && p1 > 1) {
            if ((p1 - 2) % 10)
                break;

            strcpy(path, "/usr/tycho/spool/flash/");
            strcat(path, my_userid);
            fd = open(path, O_RDONLY);
            if(fd < 1)
                break;

            len = read(fd, text, 13 * 65);
            if(len < 1) {
                blocked = TRUE;
                close(fd);
                break;
            }
            tp = text;
            text[len] = 0;
            fstat(fd, &inode);
            close(fd);
            fd = fork();

            if(!fd) {
                execv(*killflash, killflash);
                exit(-1);
            }
            else if(fd > 0)
                waitpid(fd, &len, 0);

            /* if ctrl message, abandon it */
            if(text[0] == 1)
                break;

            pwd = getpwuid(inode.st_uid);
            endpwent();
            if(pwd)
                sprintf(path, MSG_FLASH, pwd->pw_name, REAL_ID);
            else
                strcpy(path, MSG_NOTICE);

            /* if inet message, recover fully */
            if(text[0] == 2) {
                tp = strchr(text + 1, '\n');
                *(tp++) = 0;
                sprintf(path, MSG_IFLASH, text + 1);
            }

            opts->desk_notice = VUCreateQ(Notice, (STRING)path, -1, -1, tp);
            opts->desk_notice->vuparent = vu;
            opts->desk_notice->vupal = pal;
            VUTermAlert(1);
        }
    default:
        break;
    }
    return USEPORT(VUDeskPort);
}

const   DEFWINDOW(UserDesktop)
    WIN_CLASS(VUUserPort, (STRING)"Application")
    WIN_ATTR(-1, -1, HASMENUBAR, 1, NULL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Applicazione", NULL)
    REPLACE_ROMANIAN((STRING)"Program", NULL)
    REPLACE_CZECH((STRING)"Program", NULL)
#endif
ENDDEF

#endif
