/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *  Checkbox and yes/no like controls for VU.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>

#define YESCHAR yestab[replacement_language]
#define NOCHAR  notab[replacement_language]
#define TRUECHAR truetab[replacement_language]
#define FALSECHAR falsetab[replacement_language]

CONST   char    yestab[] = {'Y', 'S', 'D', 'A'};
CONST   char    notab[] =  {'N', 'N', 'N', 'N'};
CONST   char    truetab[] = {'T', 'V', 'D', 'A'};
CONST   char    falsetab[] = {'F', 'F', 'N', 'N'};

CONST   char    *truefalse[] =
{
    "TFTtftYNYyny\001",
#ifdef  BUILD_LOCALIZED
    "VFVvfvTFTYNYyny\001",
    "DNDdndYNYyny\001",
    "ANAanaYNYyny\001",
#endif
    NULL
};

DEFCONTROL(VUCheckboxControl)
{
    char    *check=(char *)control->ctext;
    CELL    cell=(vu->vupal << 8);
    int     row=control->crow + vu->vudrawn.tp;
    int     col=control->ccol + vu->vudrawn.lt;
    BOOL    flag;
    char    *p;

    switch(event) {
    case VU_KEYBOARD:
        switch(p1) {
        case ' ':
        case KEY_ENTER:
            if(control != vu->vucontrol) {
                VUControlCursor(vu,control,0,1);
                vu->vucontrol=control;
                return TRUE;
            }
            p = strchr(truefalse[replacement_language], *check);
            if(p)
                *check = *(++p);
            else if(*check)
                *check = 0;
            else
                *check = 1;
            return (*(control->cproc))(vu,control,VU_PAINT,0,0);
        case KEY_PAGEUP:
        case KEY_PAGEDOWN:
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
        case KEY_TAB:
            if(control==vu->vucontrol)
                return FALSE;
            VUControlCursor(vu, control, 0, 1);
            vu->vucontrol=control;
            break;
        default:
            return FALSE;
        }
        break;

    case VU_PAINT:

        VUDrawCell(cell | '(' ,row,col++);
        flag = TRUE;
        if(!*check || VUToLower(*check) == 'f' || VUToLower(*check) == 'n')
            flag = FALSE;
        cell += (control->cstate << 8);
        if(flag)
            VUDrawCell((cell | 'X') + 0x200,row,col++);
        else
            VUDrawCell((cell | ' ') + 0x200,row,col++);
        cell -= (control->cstate << 8);
        VUDrawCell(cell | ')',row,col);
        if(control==vu->vucontrol)
            VUControlCursor(vu,control,0,1);
        break;

    case VU_EXIT:
        control->cstate = 0;
        return (*(control->cproc))(vu, control, VU_PAINT, 0, 0);

    case VU_ENTER:
        control->cstate = 1;
        return (*(control->cproc))(vu, control, VU_PAINT, 0, 1);

    default:
        break;
    }
    return TRUE;
}

DEFCONTROL(VUYesNoControl)
{
    char    *check=(char *)control->ctext;
    CELL    cell=(vu->vupal << 8);
    int     row=control->crow + vu->vudrawn.tp;
    int     col=control->ccol + vu->vudrawn.lt;
    int     redraw=0;

    switch(event) {
    case VU_CREATE:
        if(VUToUpper(*check) == YESCHAR || VUToUpper(*check) == TRUECHAR)
            *check = YESCHAR;
        if(VUToUpper(*check) == NOCHAR || VUToUpper(*check) == FALSECHAR)
            *check = NOCHAR;
        break;
    case VU_KEYBOARD:
        if(vu->vucontrol==control) {
            if(VUToUpper(p1)==YESCHAR) {
                if(*check < 2)
                    *check = 1;
                else
                    *check = YESCHAR;
                ++redraw;
            }
            if(VUToUpper(p1)=='n') {
                if(*check < 2)
                    *check = 0;
                else
                    *check=NOCHAR;
                ++redraw;
            }
            if(redraw)
                return (*(control->cproc))(vu,control,VU_PAINT,0,0);
        }
        break;
    case VU_PAINT:
        VUDrawCell(cell | '[' ,row,col++);
        cell += (control->cstate << 8);
        if(*check && *check != NOCHAR)
            VUDrawCell(cell | (YESCHAR + 0x200),row,col++);
        else
            VUDrawCell(cell | (NOCHAR + 0x200),row,col++);
        cell -= (control->cstate << 8);
        VUDrawCell(cell | ']',row,col);
    default:
        break;
    }
    return USECONTROL(VUCheckboxControl);
}
