/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Misc. time support routines.  Often localized by national group.
 * This function contains those functions common to different national groups
 * which can if necessary be linked out by other localization routines.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>

#define vu_national vu_national_date

enum {
    ORDER_MDY = 0,
    ORDER_DMY,
    ORDER_YMD
};

CONST   char    datesep[] = {'/', '/', '.'};
CONST   int dateorder[] = {ORDER_MDY, ORDER_DMY, ORDER_DMY};

CONST   int monpos[3] = {0, 3, 3};
CONST   int daypos[3] = {3, 0, 6};
CONST   int yearpos[3] = {6, 6, 0};

FORWARD void zdig(char *buf, int val);
FORWARD int  verify(char *str);

static  void zdig(buf,val)
char    *buf;
int     val;
{
    *buf=(char)(val / 10 + '0');
    *(++buf) = (char)(val % 10 + '0');
}

char *VUCvtDate(now)
struct  tm* now;
{
    static  char    date[9] = "\?\?/\?\?/\?\?";
    int order = dateorder[vu_national];

    date[2] = date[5] = datesep[vu_national];
    zdig(date + monpos[order], now->tm_mon + 1);
    zdig(date + daypos[order], now->tm_mday);
    zdig(date + yearpos[order], now->tm_year);
    return date;
}

char    *VUBDate2Str(str)
char    *str;
{
    static  char    date[9];
    int order = dateorder[vu_national];
    int year;
    struct  tm  *dt;
    time_t  now;

    time(&now);
    dt = localtime(&now);
    year = dt->tm_year - (dt->tm_year % 100) + atoi(str + yearpos[order]);
    if(year > dt->tm_year)
        year -= 100;

    year += 1900;
    zdig(date, year / 100);
    zdig(date + 2, year % 100);
    zdig(date + 4, atoi(str + monpos[order]));
    zdig(date + 6, atoi(str + daypos[order]));
    return date;
}

char    *VUDate2Str(str)
char    *str;
{
    static  char    date[9];
    int order = dateorder[vu_national];
    int year;
    struct  tm  *dt;
    time_t  now;

    time(&now);
    dt = localtime(&now);
    year =dt->tm_year - dt->tm_year % 100 + atoi(str + yearpos[order]);
    if(year < (dt->tm_year - 50))
        year += 100;
    else if(year > (dt->tm_year + 50))
        year -= 100;

    year += 1900;
    zdig(date, year / 100);
    zdig(date + 2, year % 100);
    zdig(date + 4, atoi(str + monpos[order]));
    zdig(date + 6, atoi(str + daypos[order]));
    return date;
}

char    *VUStr2Date(str)
char    *str;
{
    static  char    date[9] = "\?\?/\?\?/\?\?";
    int order = dateorder[vu_national];

    strncpy(date + yearpos[order], str + 2, 2);
    strncpy(date + monpos[order], str + 4, 2);
    strncpy(date + daypos[order], str + 6, 2);
    date[2] = date[5] = datesep[vu_national];
    return date;
}

static  int verify(str)
char    *str;
{
    int i;

    for(i=0;i<8;++i) {
        if(i != 2 && i != 5) {
            if(str[i] < '0' || str[i] > '9')
                return i+1;
        }
    }
    return 0;
}

struct  tm* VUScanDate(mask,buf)
struct  tm* mask;
char    *buf;
{
    int order = dateorder[vu_national];
    time_t  now;
    struct  tm  *dt;
    int year;

    if(verify(buf))
        return NULL;

    time(&now);
    dt = localtime(&now);
    year = atoi(buf + yearpos[order]) + (dt->tm_year - dt->tm_year % 100);

    if (year < dt->tm_year - 50)
        year += 100;
    else if(year > dt->tm_year + 50)
        year -= 100;

    mask->tm_mon=atoi(buf + monpos[order]) - 1;
    mask->tm_mday=atoi(buf + daypos[order]);
    mask->tm_year=year;
    return mask;
}

struct  tm* VUScanBDate(mask,buf)
struct  tm* mask;
char    *buf;
{
    int order = dateorder[vu_national];
    time_t  now;
    struct  tm  *dt;
    int year;

    if(verify(buf))
        return NULL;

    time(&now);
    dt = localtime(&now);
    year = atoi(buf + yearpos[order]) + (dt->tm_year - dt->tm_year % 100);
    if(year > dt->tm_year)
        year -= 100;

    mask->tm_mon=atoi(buf + monpos[order]) - 1;
    mask->tm_mday=atoi(buf + daypos[order]);
    mask->tm_year=atoi(buf + yearpos[order]);
    if(mask->tm_year < 10)
        mask->tm_year += 100;
    return mask;
}

DEFCONTROL(VUEditDateControl)
{
    int     rtn;
    switch(event) {
    case VU_ACCEPT:
        if(0 != (rtn=verify((char *)control->ctext))) {
            control->cstate=rtn-1;
            p2=TRUE;
        }
        break;
    case VU_CREATE:
        if(!control->ctext[0])
            strcpy((char *)control->ctext,"\?\?/\?\?/\?\?");
        rtn=USECONTROL(VUEditLnControl);
        control->cstate=0;
        return rtn;
    case VU_KEYBOARD:
        if(p1==KEY_DELETE)
            return TRUE;
        if(p1==127 || p1==8)
            p1=KEY_LEFT;
        if(p1 < 32 || p1 > 127)
            break;
        if(p1 < '0' || p1 > '9') {
            if(isalpha(p1))
                return TRUE;
            else
                return FALSE;
        }
        p2=FALSE;
        switch(dateorder[vu_national]) {
        case    ORDER_MDY:

            switch(control->cstate) {
            case 0:
                if(p1 > '1')
                    return TRUE;
                break;
            case 1:
                if(control->ctext[0] == '1') {
                    if(p1 > '2')
                        return TRUE;
                }
                else if(p1=='0')
                    return TRUE;
                break;
            case 3:
                if(p1 > '3')
                    return TRUE;
            }
            break;

        case ORDER_DMY:

            switch(control->cstate) {
            case 0:
                if(p1 > '3')
                    return TRUE;
                break;
            case 4:
                if(control->ctext[3] == '1') {
                    if(p1 > '2')
                        return TRUE;
                }
                else if(p1=='0')
                    return TRUE;
                break;
            case 3:
                if(p1 > '1')
                    return TRUE;
                break;
            }
            break;


        }
        break;
    default:
        break;
    }
    return USECONTROL(VUEditMaskControl);
}
