/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>

CONST   char    *fkeys_help[] =
{
    "F1=Help",
#ifdef  BUILD_LOCALIZED
    "F1=Aiut~o",
    "F1=A~jutor",
    "F1=Napoveda",
#endif
    NULL
};

CONST   char    *fkeys_bkmark[] =
{
    "F2=B~kmark",
#ifdef  BUILD_LOCALIZED
    "F2=Indic~e",
    "F2=~Pref",
    "F2=Za~loz",
#endif
    NULL
};

CONST   char    *fkeys_page[] =
{
    "F3=~Page",
#ifdef  BUILD_LOCALIZED
    "F3=S~ysop",
    "F3=Admi~n",
    "F3=~Sysop",
#endif
    NULL
};

CONST   char    *fkeys_finger[] = {
    "F8=~Finger",
#ifdef  BUILD_LOCALIZED
    "F8=~Finger",
    "F8=Deta~lii",
    "F8=~Finger",
#endif
    NULL
};

CONST   char    *fkeys_who[] = {
    "F9=~Who",
#ifdef  BUILD_LOCALIZED
    "F9=~Who",
    "F9=~Who",
    "F9=~Kdo",
#endif
    NULL
};

static const char *fkeys[16];

const char **get_deffkeys(unsigned mode)
{
    int i = 0;
    if(mode & FKEY_HELP)
        fkeys[i++] = fkeys_help[replacement_language];

    if(mode & FKEY_BOOKMARK)
        fkeys[i++] = fkeys_bkmark[replacement_language];

    if(mode & FKEY_PAGE)
        fkeys[i++] = fkeys_page[replacement_language];

    if(mode & FKEY_FINGER)
        fkeys[i++] = fkeys_finger[replacement_language];

    if(mode & FKEY_WHO)
        fkeys[i++] = fkeys_who[replacement_language];

    fkeys[i] = NULL;
    return fkeys;
}

int get_keylabel(int keycode)
{

    switch(replacement_language) {
    case NR_ENGLISH:
        switch(keycode) {
        case 0x17:
        case ALTKEY | 'W':
            return KEY_WHO;
        case 0x06:
        case ALTKEY | 'F':
            return KEY_FINGER;
        case 0x10:
        case ALTKEY | 'P':
            return KEY_PAGE;
        case 0x0b:
        case ALTKEY | 'K':
            return KEY_BOOKMARK;
        }
        break;

#ifdef  BUILD_LOCALIZED

    case NR_ITALIAN:
        switch(keycode) {
        case 0x0f:
        case ALTKEY | 'O':
            return KEY_HELP;
        case 0x17:
        case ALTKEY | 'W':
            return KEY_WHO;
        case 0x06:
        case ALTKEY | 'F':
            return KEY_FINGER;
        case 0x19:
        case ALTKEY | 'Y':
            return KEY_PAGE;
        case 0x05:
        case ALTKEY | 'E':
            return KEY_BOOKMARK;
        }
        break;

    case NR_ROMANIAN:
        switch(keycode) {
        case 0x0a:
        case ALTKEY | 'J':
            return KEY_HELP;
        case 0x17:
        case ALTKEY | 'W':
            return KEY_WHO;
        case 0x0c:
        case ALTKEY | 'L':
            return KEY_FINGER;
        case 0x0d:
        case ALTKEY | 'N':
            return KEY_PAGE;
        case 0x10:
        case ALTKEY | 'P':
            return KEY_BOOKMARK;
        }
        break;

    case NR_CZECH:
        switch(keycode) {
        case 0x08:
        case ALTKEY | 'H':
            return KEY_HELP;
        case 0x0b:
        case ALTKEY | 'K':
            return KEY_WHO;
        case 0x06:
        case ALTKEY | 'F':
            return KEY_FINGER;
        case 0x13:
        case ALTKEY | 'S':
            return KEY_PAGE;
        case 0x0c:
        case ALTKEY | 'L':
            return KEY_BOOKMARK;
        }
        break;

#endif
    }

    switch(keycode) {
    case '?':
    case KEY_F1:
        return KEY_HELP;
    case KEY_F2:
        return KEY_BOOKMARK;
    case KEY_F3:
        return KEY_PAGE;
    case KEY_F7:
        return KEY_TALK;
    case KEY_F8:
        return KEY_FINGER;
    case KEY_F9:
        return KEY_WHO;
    case KEY_F10:
    case '!':
    case 0x26:
        return KEY_SYSREQ;
    }
    return keycode;
}

