/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    This procedure handles the default desktop properites under VU.

    0.4     David Sugar     02 20 92

    If speed below 4800bps, desktop pattern disabled.
*/

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>

CONST   char    *exitmsg[] =
{
    "Exit ",
#ifdef  BUILD_LOCALIZED
    "Esci da ",
    "Abandon ",
    "Ukoncit ",
#endif
    NULL
};

DEFPORT(VUDeskPort)
{
    static  int top=0;
    int row,col;
    static  time_t  t,t1=0l;
    struct  tm  *now;
    char    *stdtime;
    static  char    exitname[40];
    DESKOPTS opts = (DESKOPTS)vu->vuoptions;
    VU child;

    switch(event) {
    case VU_NOTIFY:
        if(p1 == VU_SHUTDOWN) {
            opts->desk_shutdown = FALSE;
            if(p2 == VU_ACCEPT)
                VUPostEvent(desktop,VU_SHUTDOWN,0,0);
        }
        break;
    case VU_BLANK:
        VUTermBlank();      /* fall thru idle */
    case VU_IDLE:
        if(top) {
            time(&t);

            if((t / 60)  != (t1 / 60)) {
                t1=t;
                now=localtime(&t);
                stdtime=VUCvtTime(now);
                stdtime[5] = 0; /* strip a/pm */
                col=vu->vudrawn.rt - 5;
                row=vu->vudrawn.tp;
                VUDrawAbsCell(*(stdtime++) | 0x0500, row, col++);
                VUDrawAbsCell(*(stdtime++) | 0x0500, row, col++);
                VUDrawAbsCell(*(stdtime++) | 0x0600, row, col++);
                VUDrawAbsCell(*(stdtime++) | 0x0500, row, col++);
                VUDrawAbsCell(*stdtime | 0x0500, row, col);
                VUPostRedraw(VU_REFRESH,NULL,NULL);
            }
        }
        return FALSE;
    case VU_PAINT:
        p2=deskpattern;
        break;
    case VU_CREATE:
        if(desktop != NULL)     /* only 1 desktop per app. */
            return FALSE;
        desktop=vu;
        opts->desk_shutdown = FALSE;
        opts->desk_locked = FALSE;
        opts->desk_ntext = NULL;
        opts->desk_notice = NULL;
        if(!color || speed < 4800)
            deskpattern=0x0020;
        if(!(VUWindowPort(vu,VU_CREATE,0,deskpattern)))
            return FALSE;
        VUWindowPort(vu,VU_PAINT,0,deskpattern);    /* paint desktop */
        viewport=vu->vudrawn;
        if(VUGetAttr(vu,HASMENUBAR)) {
            ++top;
            viewport.tp=vu->vudrawn.tp + 1;
        }
        if(VUGetAttr(vu,HASBORDER))
            viewport.tp=vu->vudrawn.tp + 1;
        VUToFront(vu);              /* push to front and draw */
        VUSetAttr(vu,LOCKED|ACTIVE);        /* lock desktop */
        VUEvent(desktop,VU_IDLE,0,0);
        return TRUE;
    case VU_KEYBOARD:
        switch(p1) {
        case KEY_REFRESH:
        case 18:
            VUForceRedraw();
            return TRUE;
        case KEY_EXIT:
            if(vu != front)         /* esc only if at top */
                return FALSE;

        case KEY_SYSREQ:
        case 3:
            strcpy(exitname, exitmsg[replacement_language]);
            strcat(exitname, (char *)vu->vutitle);
            strcat(exitname,"?");
            child = VUYesNoBox(desktop,(STRING)exitname);
            child->vuresid = VU_SHUTDOWN;
            opts->desk_shutdown = TRUE;
            return TRUE;
        default:
            break;
        }
        break;
    case VU_REDRAW:             /* sending redraw to desktop */
        VUForceRedraw();
        return TRUE;
    case VU_SHUTDOWN:
        ++shutdown_flag;
        break;
    case VU_DESTROY:
        if(p2==TRUE) {
            if(opts->desk_ntext)
                free(opts->desk_ntext);
            VUDestroy(vu);
        }
        return FALSE;                   /* desktop cannot shutdown ever */
    default:
        break;
    }
    return USEPORT(VUWindowPort);
}

const   DEFWINDOW(VUDesktop)
    WIN_CLASS(VUDeskPort, (STRING)"Application")
    WIN_ATTR(-1, -1, HASMENUBAR, 1, NULL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Applicazione", (STRING)NULL)
    REPLACE_ROMANIAN((STRING)"Program", (STRING)NULL)
    REPLACE_CZECH((STRING)"Program", (STRING)NULL)
#endif
ENDDEF
