/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/edit.h>

int VUScrollPad(VU vu, int lines, BOOL mode)
{
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE *base = (EDLINE *)vu->vupaint;
    EDLINE *ln = (EDLINE *)ctrl->ed_current;
    int count = lines;
    int adj = 1;

    if(count < 0) {
        adj = -1;
        count = -count;
    }

    while(count--) {
        if(lines < 0) {
            if(!base->ed_prev)
                break;

            base = base->ed_prev;
            if(mode) {
                ln = ln->ed_prev;
                ctrl->ed_clineno += adj;
            }
        }
        else {
            if(!base->ed_next)
                break;

            base = base->ed_next;
            if(ln->ed_next && mode) {
                ln = ln->ed_next;
                ctrl->ed_clineno += adj;
            }
        }
        opts->ed_lineno += adj;
    }
    ctrl->ed_current = ln;
    vu->vupaint = (char *)base;
    VUUpdatePad(vu);
    vu->vustatline = (STRING)ctrl->ed_statline;
    vu->vustatcolor = STAT_INFO;
    return count;
}

int VUAddEditText(VU vu, char mode, char *text)
{
    EDCTRL *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE *line, *next;
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;
    int len = strlen(text);

    if(!mode)
        mode = vu->vupal;

    if(!ctrl->ed_free)
        return 0;

    if(len > opts->ed_width)
        len = opts->ed_width;
    line = ctrl->ed_last;
    next = ctrl->ed_free;
    ctrl->ed_free = next->ed_next;
    next->ed_next = NULL;
    if(line) {
        next->ed_prev = line;
        line->ed_next = next;
        ++ctrl->ed_llineno;
    }
    else {
        next->ed_prev = NULL;
        ctrl->ed_first = next;
        ctrl->ed_current = next;
        opts->ed_lineno = ctrl->ed_clineno = ctrl->ed_llineno = 1;
        vu->vupaint = (char *)next;
    }
    ctrl->ed_last = next;
    next->ed_mask = mode;
    strncpy(next->ed_data, text, len);
    next->ed_data[len] = 0;
    VUUpdatePad(vu);
    return 1;
}

int VUNewEditText(VU vu, char mode)
{
    EDCTRL *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE *line, *next;
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;

    if(!mode)
        mode = vu->vupal;
    line = ctrl->ed_first;
    while(line) {
        next = line->ed_next;
        line->ed_next = ctrl->ed_free;
        ctrl->ed_free = line;
        line = next;
    }
    line = ctrl->ed_free;
    ctrl->ed_free = line->ed_next;
    line->ed_next = line->ed_prev = NULL;
    ctrl->ed_first = ctrl->ed_last = ctrl->ed_current = line;
    line->ed_mask = mode;
    line->ed_data[0] = 0;
    opts->ed_lineno = ctrl->ed_clineno = ctrl->ed_llineno = 1;
    vu->vupaint = (char *)line;
    vu->vucurrow = 0;
    if(VUGetAttr(vu, HASBORDER) || VUGetAttr(vu, HASMENUBAR))
        ++vu->vucurrow;
    vu->vucurcol = opts->ed_number;
    if(VUGetAttr(vu, HASBORDER))
        ++vu->vucurcol;
    VUUpdatePad(vu);
    return 1;
}

