/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>
#include <worldvu/edit.h>

#ifndef MAXEDIT
#define MAXEDIT 60000u
#endif

DEFPORT(VUEditPad)
{
    EDOPTS  *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL  *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE  *ln, *base;
    int lines;
    int left = 0;
    int len;
    long    max;

    if(VUGetAttr(vu, HASBORDER))
        ++left;

    switch(event) {
    case VU_PAINT:
        if(ctrl->ed_current) {
            vu->vucurrow = ctrl->ed_clineno - opts->ed_lineno;
            if(left || VUGetAttr(vu, HASMENUBAR))
                ++vu->vucurrow;
            len = strlen(ctrl->ed_current->ed_data);
            if(opts->ed_left > len)
                opts->ed_left = 0;
            vu->vucurmode = 1;
            if((vu->vucurcol - left - opts->ed_number) > (len - opts->ed_left))
                vu->vucurcol = len - opts->ed_left;

            max = vu->vudrawn.rt - vu->vudrawn.lt - left;

            if(vu->vucurcol > max) {
                opts->ed_left = vu->vucurcol - left - 8;
                vu->vucurcol = left + 8;
            }
        }
        else
            vu->vucurmode = 0;
        break;
    case VU_KEYBOARD:
        if(vu != front)
            break;

        lines = vu->vudrawn.bt - vu->vudrawn.tp + 1;
        if(VUGetAttr(vu, HASBORDER))
            lines -=2;
        else if(VUGetAttr(vu, HASMENUBAR))
            lines -=1;

        base = (EDLINE *)vu->vupaint;
        ln = ctrl->ed_current;

        switch(p1) {
        case 2:
            vu->vucurcol = left + opts->ed_number;
            if(opts->ed_left) {
                opts->ed_left = 0;
                VUUpdatePad(vu);
            }
            else
                fixcursor();
            return TRUE;
        case KEY_HOME:
            ctrl->ed_current = ctrl->ed_first;
            opts->ed_lineno = ctrl->ed_clineno = 1;
            vu->vupaint = (char *)ctrl->ed_first;
            VUUpdatePad(vu);
            return TRUE;
        case KEY_END:
            ctrl->ed_current = ctrl->ed_last;
            vu->vupaint = (char *)ctrl->ed_last;
            opts->ed_lineno = ctrl->ed_clineno = ctrl->ed_llineno;
            VUScrollPad(vu, - (--lines), FALSE);
            return TRUE;
        case 0x10:
        case KEY_PAGEUP:
            VUScrollPad(vu, - (--lines), TRUE);
            return TRUE;
        case 0x0e:
        case KEY_PAGEDOWN:
            VUScrollPad(vu, --lines, TRUE);
            return TRUE;
        case KEY_RIGHT:
            if(!ln->ed_data[vu->vucurcol - left - opts->ed_number + opts->ed_left]) {
                if(!ln->ed_next)
                    return TRUE;
                ln = ln->ed_next;
                ++ctrl->ed_clineno;
                ctrl->ed_current = ln;
                vu->vucurcol = opts->ed_number + left;
                if((ctrl->ed_clineno - opts->ed_lineno) >= lines)
                    VUScrollPad(vu, lines / 3, FALSE);
                else
                    VUUpdatePad(vu);
            }
            else {
                max = vu->vudrawn.rt - vu->vudrawn.lt - left;
                if(++vu->vucurcol >= max) {
                    opts->ed_left += 8;
                    vu->vucurcol -= 8;
                    VUUpdatePad(vu);
                }
                else
                    fixcursor();
            }
            return TRUE;
        case KEY_LEFT:
            if(vu->vucurcol <= (left + opts->ed_number)) {
                if(opts->ed_left) {
                    max = 8;
                    vu->vucurcol = left - 1;
                    while(opts->ed_left && max--) {
                        --opts->ed_left;
                        ++vu->vucurcol;
                    }
                }
                else {
                    if(!ln->ed_prev)
                        return TRUE;

                    if(ln == base) {
                        VUScrollPad(vu, - lines / 3, FALSE);
                        base = (EDLINE *)vu->vupaint;
                        ln = ctrl->ed_current;
                        --opts->ed_lineno;
                    }
                    ln = ln->ed_prev;
                    --ctrl->ed_clineno;
                    vu->vucurcol = strlen(ln->ed_data);
                    ctrl->ed_current = ln;
                }
                VUUpdatePad(vu);
            }
            else {
                --vu->vucurcol;
                fixcursor();
            }
            return TRUE;
        default:
            if(p1 > 255)
                break;

            if(!VUValidText(p1))
                break;

            if(ln->ed_mask & ED_READONLY)
                return TRUE;
            max = vu->vucurcol - left - opts->ed_number + opts->ed_left;
            len = strlen(ln->ed_data);
            if(len >= opts->ed_width)
                return TRUE;
            if(max >= len) {
                ln->ed_data[len++] = (char)p1;
                ln->ed_data[len] = 0;
            }
            else if(insert_mode) {
                while(len >= max) {
                    ln->ed_data[len + 1] = ln->ed_data[len];
                    --len;
                }
                ln->ed_data[max] = (char)p1;
            }
            else
                ln->ed_data[max] = (char)p1;
            ++vu->vucurcol;
            VUUpdatePad(vu);
            return TRUE;
        }
        break;
    case VU_CREATE:
        if(!USEPORT(VUTextPad))
            return FALSE;

        max = MAXEDIT;

        if(p1 > TRUE) {
            if((unsigned)p1 < MAXEDIT)
                max = p1;
            p1 = TRUE;
        }
        if(p2 > TRUE) {
            opts->ed_width = p2 / 2;
            p2 = p2 & 0x01;
        }
        vu->vuimage = malloc(sizeof(EDCTRL) + max);
        ctrl = (EDCTRL *)vu->vuimage;
        ln = (EDLINE *)((char *)vu->vuimage + sizeof(EDCTRL));
        if(!ctrl)
            return FALSE;
        lines = max / (opts->ed_width + 1 + sizeof(EDLINE));
        ctrl->ed_free = ln;
        ctrl->ed_current = NULL;
        ctrl->ed_first = NULL;
        ctrl->ed_last = NULL;
        ctrl->ed_clineno = 0;
        ctrl->ed_llineno = 0;
        ctrl->ed_statline = (char *)vu->vustatline;
        lines -= 2;
        while(--lines) {
            ln->ed_mask = 0;
            ln->ed_next = (EDLINE *)((char *)ln + opts->ed_width + 1 + sizeof(EDLINE));
            ln = ln->ed_next;
        }
        ln->ed_next = NULL;
        vu->vucurcol = 0;
        ctrl->ed_wrap = (char *)ln + opts->ed_width + 1 + sizeof(EDLINE);
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUTextPad);
}
