/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    Failure message window.

    0.5     David Sugar     Feb 23 92

    Keeps status line from parent (front) task.
*/

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>

WINDOW(FailMsg);

VU VUFailMsg(char *str, int row, int col)
{
    register    int len;
    int i;

    len = strlen(str);
    FailMsg[1].ctext = (STRING)str;
    if(len > 26) {
        FailMsg[0].cmsg = front->vustatline;    /* keep status line alive */
        FailMsg[0].cwid += (len - 26);
        FailMsg[1].cwid = len;
        len = (len - 26) / 2;
        for(i = 0; i < NR_COUNT; ++i) {
            FailMsg[2 + i * 3].ccol += len;
            FailMsg[3 + i * 3].ccol += len;
        }
    }
    return VUCreateQ(FailMsg, NULL, row, col, NULL);
}

DEFWINDOW(ErrorMsg)
    WIN_CLASS(VUMsgboxPort, NULL)
    WIN_ATTR(5, 30, HASSHADOW | HASBORDER, 32, (STRING)"Press <ESCAPE> to continue")

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, (STRING)"Premi <ESC> per continuare")
    REPLACE_ROMANIAN(NULL, (STRING)"Aphasti <ESCAPE> pentru terminare")
    REPLACE_CZECH(NULL, (STRING)"Chcete-li pokracovat, stisknete <ESC>")
#endif
ENDDEF

DEFWINDOW(FailMsg)
    WIN_CLASS(VUFlashPort,NULL)
    WIN_ATTR(6, 30, LOCKED | HASSHADOW | HASBORDER, 32, NULL)

    TEXT_CONTROL  (NULL,    1, 2, 1, 26, 0)
    REPLACE_ENGLISH(NULL, NULL)
    BUTTON_CONTROL((STRING)"~Retry ", 3, 6, 6, VU_RETRY)
    BUTTON_CONTROL((STRING)" ~Fail ", 3, 18, 6, VU_FAIL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, NULL)
    BUTTON_CONTROL((STRING)"~Riprova ", 3, 6, 8, VU_RETRY)
    BUTTON_CONTROL((STRING)"  ~Esci  ", 3,18, 8, VU_FAIL)
    REPLACE_ROMANIAN(NULL, NULL)
    BUTTON_CONTROL((STRING)"~Din nou ", 3, 6, 8, VU_RETRY)
    BUTTON_CONTROL((STRING)"~Abandon ", 3,18, 8, VU_FAIL)
    REPLACE_CZECH(NULL, NULL)
    BUTTON_CONTROL((STRING)"Z~novu ", 3, 6, 8, VU_RETRY)
    BUTTON_CONTROL((STRING)"~Zrusit", 3,18, 8, VU_FAIL)
#endif
ENDDEF
