/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    Flash window is used normally to place a message IMMEDIATELY on the screen.
    Flash window update occurs with creation and destruction.

    Revised 0.6     David Sugar     March 9 92

    During create the image is flushed immediately to screen if paint content
    is set.

*/

#include <vu-config.h>
#include <worldvu/vu.h>

DEFPORT(VUFlashPort)
{
    int rtn;

    switch(event) {
    case VU_CREATE:
        if(vu->vupaint != NULL)
            VUFlush();
        VUSetAttr(vu,BLOCKED);
        event = VU_PAINT;
    case VU_PAINT:
        if(vu != front) {
            if(vu == back) {
                back = vu->vuprev;
                back->vunext = NULL;
            }
            else {
                vu->vuprev->vunext = vu->vunext;
                vu->vunext->vuprev = vu->vuprev;
            }
            front->vuprev = vu;
            vu->vunext = front;
            front = vu;
            vu->vuprev = NULL;
        }
        rtn = USEPORT(VUMsgboxPort);
        if(rtn) {
            VUFlush();
            fixcursor();
        }
        return rtn;
    case VU_DESTROY:
        VUDestroy(vu);
        VUFlush();
        return TRUE;
    default:
        return USEPORT(VUMsgboxPort);
    }
}

void VUFlashMessage(VU vu, char *msg)
{
    vu->vupaint = msg;
    VUEvent(vu, VU_PAINT, 0, 0);
}

