/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    The VU help system operates through a special help window
    and message port.  The help window is originally created
    BEHIND the desktop so that it remains invisible, and is
    marked as ACTIVE.

    When the VU help window receives an F1 key event, it examines
    the res id of the window currently in front, and the event id
    of it's control item.  This information is used to search
    through the application's help file.  The help window is then
    placed at the front with the help information retreived.

    When the user presses <ESCAPE>, the help window moves back to
    behind the desktop.

    The create call will pass the name of the help file to use in
    vupaint.  The help system may search a pre-planned master help
    directory, depending on OS specific implimentations.

    Your application should normally create only one help window.
    If you want to use multiple help windows which appear in different
    places on the screen, you can do so, so long as you pass different
    filenames when you create each help window.
*/

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/help.h>

#ifdef  __MSDOS__
#define FMODE   "rb"
#endif

#ifdef  QNX
#define FMODE   "rc"
#endif

#ifndef FMODE
#define FMODE   "r"
#endif

DEFPORT(VUHelpFilePort)
{
    typedef struct {
        FILE    *fp;
        char    *fn;
        short   index_size;
        short   move_size;
    } *HELP;
    char    buf[128];

    RECT    rc;

    short   middle,mode,test;
    char    *env;

    HELP hp = (HELP)vu->vuoptions;
    char    *text = vu->vupaint;
    short   i;
#ifdef  __SMALL__
    static HELP_INDEX entry;
    HELP_INDEX *help = &entry;
#else
    HELP_INDEX *help = (HELP_INDEX *)vu->vuimage;
#endif

    short   window,item;

    switch(event) {
    case VU_CREATE:
        env = getenv("LANG");
        if(env) {
            strcpy(buf, env);
            env = strtok(buf, "_.");
        }
        else
            env = "default";

        hp->fn = (char *)malloc(64);
        strcpy(hp->fn, PREFIX_DATADIR "/");
        strcat(hp->fn, env);
        strcat(hp->fn, "/");
        strcat(hp->fn, vu->vupaint);
        strcat(hp->fn, ".help");

        if(NULL == (hp->fp=fopen(hp->fn, FMODE))) {

            strcpy(hp->fn, PREFIX_DATADIR "/default/");
            strcat(hp->fn, vu->vupaint);
            strcat(hp->fn, ".help");
            if(NULL == (hp->fp = fopen(hp->fn, FMODE)))
                return FALSE;
        }

        fread((char *)&hp->index_size,1,sizeof(short),hp->fp);
#ifndef  __SMALL__
        if(NULL == (vu->vuimage = (PTR)malloc(hp->index_size * sizeof(HELP_INDEX))))
            return FALSE;
        fread(vu->vuimage,sizeof(HELP_INDEX), hp->index_size, hp->fp);
#endif
        if(NULL == (vu->vupaint = (char *)malloc(1400)))
            return FALSE;
        *((char *)vu->vupaint) = 0;
        VUSetAttr(vu, ACTIVE);
        p2 = TRUE;
        hp->move_size = 3;
        fclose(hp->fp);
        hp->fp = NULL;
        break;
    case VU_HELP:
        if(vu == front)
            return FALSE;

        window = p1;
        item = p2;
        goto    dohelp;

    case VU_KEYBOARD:
        if(KEY_HELP == get_keylabel(p1)) {
            if(vu == front)
                return FALSE;

            window = front->vuresid;
            item = front->vucontrol->ctag;

            window = front->vuresid;
            item = front->vucontrol->ctag;

dohelp:
            memcpy(&vu->vudrawn, &vu->vuorigin, sizeof(RECT));
            middle = (viewport.bt - viewport.tp) / 2 + viewport.tp;
            mode = 0;
            if(vu->vudrawn.bt < middle)
                mode = middle - vu->vudrawn.bt;
            if(vu->vudrawn.tp > middle)
                mode = middle - vu->vudrawn.tp;
            test = middle - (front->vucurrow + front->vudrawn.tp);

            if(mode < 0 && test < 0) {
                test = vu->vudrawn.bt - vu->vudrawn.tp;
                vu->vudrawn.bt = middle + mode;
                vu->vudrawn.tp = middle + mode - test;
            }

            if(mode > 0 && test > 0) {
                test = vu->vudrawn.bt - vu->vudrawn.tp;
                vu->vudrawn.tp = middle + mode;
                vu->vudrawn.bt = middle + mode + test;
            }
            if(NULL == hp->fp)
                hp->fp = fopen(hp->fn,FMODE);
            if(NULL == hp->fp)
                return FALSE;
#ifndef  __SMALL__
            for(i=0;i<hp->index_size;++i) {
                if(help->help_resid == window || !help->help_resid)
                    if(help->help_event == item)
                        break;
                ++help;
            }
#else
            fseek(hp->fp, 2l, SEEK_SET);
            for(i = 0; i< hp->index_size; ++i) {
                fread((char *)help, sizeof(entry), 1, hp->fp);
                if(!help->help_resid || help->help_resid == window)
                    if(help->help_event == item)
                        break;
            }
#endif
            if(i == hp->index_size)
                return TRUE;

            vu->vutitle = (STRING)help->help_title;
            fseek(hp->fp, help->help_offset, SEEK_SET);
            fread(text, help->help_length, 1, hp->fp);
            vu->vucurrow = front->vucurrow + front->vudrawn.tp - vu->vudrawn.tp;
            vu->vucurcol = front->vucurcol + front->vudrawn.lt - vu->vudrawn.lt;
            vu->vucurmode = front->vucurmode;
            if(strlen(text) > 400) {
                vu->vudrawn.tp = viewport.tp + 2;
                vu->vudrawn.bt = viewport.bt - 2;
            }
            VUToFront(vu);
            return TRUE;
        }

        if(vu != front)
            return FALSE;

        memcpy(&rc, &vu->vudrawn, sizeof(rc));

        switch(p1) {
        case KEY_EXIT:
            VUToBack(vu);
            return TRUE;

        case KEY_UP:
            rc.tp -= hp->move_size;
            rc.bt -= hp->move_size;
            vu->vucurrow += hp->move_size;
            VUChangeRect(vu, &rc);
            return TRUE;

        case KEY_DOWN:
            rc.tp += hp->move_size;
            rc.bt += hp->move_size;
            vu->vucurrow -= hp->move_size;
            VUChangeRect(vu, &rc);
            return TRUE;

        case KEY_LEFT:
            rc.lt -= hp->move_size;
            rc.rt -= hp->move_size;
            vu->vucurcol += hp->move_size;
            VUChangeRect(vu,&rc);
            return TRUE;

        case KEY_RIGHT:
            rc.lt += hp->move_size;
            rc.rt += hp->move_size;
            vu->vucurcol -= hp->move_size;
            VUChangeRect(vu,&rc);
            return TRUE;
        }
        break;
    case VU_IDLE:
        if(p1 > 10)
            if(hp->fp) {
                fclose(hp->fp);
                hp->fp = NULL;
            }

        if(!p1)
            break;

        if(p1 != help_timer)
            break;

        if(vu == front)
            break;

        VUPostEvent(vu, VU_KEYBOARD, KEY_HELP, 0);
        break;
    case VU_DESTROY:
        if(hp->fp != NULL)
            fclose(hp->fp);
        free(vu->vupaint);
        free(hp->fn);
#ifdef  _BIG_DATA_
        free(vu->vuimage);
        vu->vuimage=NULL;
#endif
        break;
    default:
        break;
    }
    return USEPORT(VUMsgboxPort);
}

