/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.	If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Routines to manipulate imaginary screen memory
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/queue.h>
#include <worldvu/multi.h>

extern	BOOL	shadow_button;
extern	ATTR	attrmask;

static void shadow(int r, int c);

void VUImageFill(RECT *rc, CELL cell)
{
    register	CELL		*mark;
    int			c,r;
	
    if(rc->rt < viewport.lt || rc->lt > viewport.rt || rc->tp > viewport.bt || rc->bt < viewport.tp)
	return;
    if(rc->rt > viewport.rt)
	rc->rt = viewport.rt;
    if(rc->lt < viewport.lt)
	rc->lt = viewport.lt;
    if(rc->tp < viewport.tp)
	rc->tp = viewport.tp;
    if(rc->bt > viewport.bt)
	rc->bt = viewport.bt;
    for(r = rc->tp; r <= rc->bt; ++r) {
	mark=imageport + r * cols + rc->lt;
	for(c = rc->lt; c <= rc->rt; ++c) {
	    *mark = cell;
	    ++mark;
	}
    }
}

void VUDrawAbsCell(CELL cell, int r, int c)
{
    register	CELL	*mark;

    if(r < 0 || r >= rows || c < 0 || c >= cols) 
	return;

    mark=imageport + cols * r + c;
    *mark = cell;
}
		
void VUDrawCell(CELL cell, int r, int c)
{
    register	CELL	*mark;

    if(r < viewport.tp || r > viewport.bt || c < viewport.lt || c > viewport.rt)
	return;

    mark=imageport + cols * r + c;
    *mark = cell;
}

void VUPressButton(CITEM *c, VU vu)
{
    register	CITEM	*control=c;

    int row, col, wid;
    ATTR cell;
    int w1, c1, shadow = (int)vu->vupal;

    row = control->crow+vu->vudrawn.tp;
    col = control->ccol+vu->vudrawn.lt;
    wid = control->cwid;

    w1 = wid;
    c1 = col;

    if(!shadow_button)
	shadow = 0;

    while(wid--)	
	VUDrawCell(0x8e20, row, col++);

    if(shadow) {
	cell = (shadow << 8) | 0x0020;
	VUDrawCell(cell, row, col);
	col = c1;
	wid = w1;
	++row;
	while(w1--)
	    VUDrawCell(cell, row, ++c1);
	--row;
    }
    else {
	col = c1;
	wid = w1;
    }

    VUTermUpdate();
    if(!VUTermPending())
	sync_timer(8);
    VUDrawButtonUp(control->ctext, row, col, wid, shadow);
    VUTermUpdate();
}
	
void VUDrawButtonUp(STRING str, int row, int col,int wid, PAL shadow)
{
    register ATTR cell = 0x0E00;
    int w1 = wid, c1 = col;
    int len = strlen((char *)str);

    /*	we copy str to buf because str may be in constant memory,
	forcing unexpected memory violations in some protected
	mode operating systems	
     */

    str = VUTextCtrlString(str);

    if(!shadow_button) {
	if(*str == ' ' && str[len - 1] == ' ') {
	    *str='[';
	    str[len - 1] = ']';
	}
	shadow=0;
    }

    while(wid--) {
	if(*str) {
	    if(*str == VU_MARKER_CELL)	
		VUDrawCell(0x0F00 | (CELL)*(++str), row, col);
	    else
		VUDrawCell(cell | (CELL)*str, row, col);

	    ++str;
	}
	else
	    VUDrawCell(cell | 0x20, row, col);
	++col;
    }
    cell = shadow << 8 | GRAPHIC_CELL;
    if(shadow) {
	VUDrawCell(cell | BOX_FILL_BOT, row, col);
	++row;
	while(w1--)
	    VUDrawCell(cell | BOX_FILL_TOP, row, ++c1);
    }
}

static void shadow(int r, int c)
{
    register CELL *mark;
    register unsigned pal;	

    if(r < viewport.tp || r > viewport.bt || c < viewport.lt || c > viewport.rt)
	return;

    mark = imageport + cols * r + c;
    pal = (*mark & PALMASK_CELL) >> 8;
    if(palette[pal].palshadow == pal)
	return;
    else
	*mark = (*mark &~ PALMASK_CELL) | (palette[pal].palshadow << 8);
}

void VUDrawString(STRING str, int r, int c, int max, CELL pal)
{
    pal = pal << 8;

    while(max--) {
	if(*str)
	    VUDrawCell(pal | *(str++), r, c);
	else
	    VUDrawCell(pal | 0x0020, r, c);
	++c;
    }
}

void VUDrawFrame(VU vu, CELL pal)
{
    register int r,c;

    pal = pal << 8 | GRAPHIC_CELL;		/* shift over to attr table */

    VUDrawCell(pal | BOX_D|BOX_R, vu->vudrawn.tp, vu->vudrawn.lt);
    VUDrawCell(pal | BOX_D|BOX_L, vu->vudrawn.tp, vu->vudrawn.rt);
    VUDrawCell(pal | BOX_U|BOX_R, vu->vudrawn.bt, vu->vudrawn.lt);
    VUDrawCell(pal | BOX_U|BOX_L, vu->vudrawn.bt, vu->vudrawn.rt);

    if(VUGetAttr(vu, HASTITLE)) {
	VUDrawCell(pal | BOX_L|BOX_U|BOX_D, vu->vudrawn.tp, vu->vudrawn.lt + 1);
	VUDrawCell(pal | BOX_R|BOX_U|BOX_D, vu->vudrawn.tp, vu->vudrawn.lt + 2 + strlen((char *)vu->vutitle));
	for(c = vu->vudrawn.lt + 3 + strlen((char *)vu->vutitle); c < vu->vudrawn.rt; ++c) {
	    VUDrawCell(pal | BOX_L|BOX_R, vu->vudrawn.tp, c);
	}
    }
    else if(!VUGetAttr(vu, HASMENUBAR)) {				
	r = vu->vudrawn.tp;		
	for(c=vu->vudrawn.lt + 1; c< vu->vudrawn.rt;++c)
	VUDrawCell(pal | BOX_L|BOX_R, r, c);
    }

    r = vu->vudrawn.bt;
    for(c=vu->vudrawn.lt + 1; c < vu->vudrawn.rt; ++c)
	VUDrawCell(pal | BOX_L|BOX_R, r, c);
    for(r=vu->vudrawn.tp + 1; r < vu->vudrawn.bt; ++r) {
	VUDrawCell(pal | BOX_U|BOX_D, r, vu->vudrawn.lt);
	VUDrawCell(pal | BOX_U| BOX_D, r, vu->vudrawn.rt);
    }

    if(color && VUGetAttr(vu, HASSHADOW)) {
	c = vu->vudrawn.rt + 1;
	for(r = vu->vudrawn.tp + 1;r <= vu->vudrawn.bt + 1; ++r) {
	    shadow(r,c);
	    shadow(r,c+1);
	}
	r = vu->vudrawn.bt+1;
	for(c = vu->vudrawn.lt + 1;c <= vu->vudrawn.rt; ++c)
	    shadow(r,c);
    }
}

void VUForceRedraw()
{
    int c;
    int	offset = rows * cols;

    for(c = 0; c < offset; ++c)
	*(imageport + offset + c) = 0;
    
    VUPostRedraw(VU_REFRESH, NULL, NULL);
}



