/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*  Julian date conversion for VU database storage. */

#include <vu-config.h>
#include <time.h>
#include <worldvu/vu.h>
#include <worldvu/julian.h>

julian_t mkjulian(struct tm *date)
{
    long    year = (long)date->tm_year + 1900l;
    long    month = (long)date->tm_mon + 1l;
    long    day = (long)date->tm_mday;

    return (julian_t)(day - 32075L + 1461L * (year + 4800L + (month - 14L) / 12L) / 4L +
        367L * (month - 2L - (month - 14L) / 12L * 12L) / 12L -
        3L * ((year + 4900L + (month - 14L) / 12L) / 100L) / 4L);
}

struct  tm *julian(julian_t jdate)
{
    static  struct  tm  date;
    long    t1, t2, year, month;

    date.tm_hour = 0;
    date.tm_min = 0;
    date.tm_sec = 0;

    t1 = jdate + 68569L;
    t2 = 4L * t1 / 146097L;
    t1 = t1 - (146097L * t2 + 3L) / 4L;
    year  = 4000L * (t1 + 1) / 1461001L;
    t1 = t1 - 1461L * year / 4L + 31L;
    month = 80L * t1 / 2447L;
    date.tm_mday  = (int)(t1 - 2447L * month / 80L);
    t1 = month / 11L;
    date.tm_mon   = (int)(month + 1L - 12L * t1);
    date.tm_year  = (int)(100L * (t2 - 49L) + year + t1) - 1900;

    return &date;
}


