/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/edit.h>

DEFPORT(VUMemoCtrlPad)
{
    EDCTRL  *ctrl;
    EDLINE  *ln;
    int     left  = 0;

    if(VUGetAttr(vu, HASBORDER))
        ++left;

    switch(event) {
    case VU_FRONT:
        if(p1) {
            (*vu->vuparent->vuproc)(vu->vuparent, event, p1, p2);
            VUSetAttr(vu->vuparent, ACTIVE);
        }
        else {
            VUClrAttr(vu->vuparent, ACTIVE);
            if(vu->vuparent != front)
                (*vu->vuparent->vuproc)(vu->vuparent, event, p1, p2);
        }
        break;
    case VU_DESTROY:
        if(!p2) {
            VUPostEvent(vu->vuparent, event, p1, p2);
            return TRUE;
        }
        break;
    case VU_KEYBOARD:
        if(vu != front)
            return FALSE;

        ctrl = (EDCTRL *)vu->vuimage;
        ln = ctrl->ed_current;

        switch(p1) {
        case KEY_LEFT:
            if(vu->vucurcol > left)
                break;
        case KEY_UP:
            if(ln->ed_prev)
                break;
        case 0x09:
            vu->vustatline = (STRING)ctrl->ed_statline;
            VUToFront(vu->vuparent);
            VUPostEvent(vu->vuparent, event, p1, p2);
            return TRUE;
        }
    default:
        break;
    }
    return USEPORT(VUMemoPad);
}

DEFCONTROL(VUMemoControl)
{
    BOOL active = FALSE;

    if(vu->vucontrol == control)
        active = TRUE;

    switch(event) {
    case VU_PAINT:
        if(control->cstate)
            if((VU)control->cmsg != front)
                (*((VU)control->cmsg)->vuproc)((VU)control->cmsg, VU_PAINT, 0, 0);
        return TRUE;
    case VU_DESTROY:
        VUPostEvent((VU)control->cmsg, VU_DESTROY, 0, TRUE);
        break;
    case VU_CREATE:
        if(!(control->cmsg = (STRING)VUCreate((CITEM *)control->ctext, NULL,
          control->crow + vu->vudrawn.tp, control->ccol + vu->vudrawn.rt,
          VUNULL, 0xffff, vu, NULL)))
            return FALSE;
        control->cstate = FALSE;
        return TRUE;
    case VU_KEYBOARD:
        switch(p1) {
        case KEY_DOWN:
        case KEY_UP:
        case KEY_LEFT:
        case KEY_RIGHT:
        case KEY_ENTER:
        case KEY_PAGEDOWN:
        case KEY_PAGEUP:
        case KEY_TAB:
            if(active)
                return FALSE;
            vu->vucontrol = control;
            control->cstate = TRUE;
            VUToFront(((VU)control->cmsg));
            return TRUE;
        }
        return FALSE;
    default:
        break;
    }
    return TRUE;
}
