/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>
#include <worldvu/edit.h>

#ifndef MAXEDIT
#define MAXEDIT 60000
#endif

#define MSG_MEM     (char *)msgs[replacement_language * 4]
#define MSG_EMPTY   (char *)msgs[replacement_language * 4 + 1]
#define MSG_EOT     (char *)msgs[replacement_language * 4 + 2]
#define MSG_BOT     (char *)msgs[replacement_language * 4 + 3]

CONST char *msgs[] =
{
    "*** out of memory ***",
    "*** scratchpad is empty ***",
    "*** end of text ***",
    "*** top of text ***",

#ifdef  BUILD_LOCALIZED
    "*** memoria insufficiente ***",
    "*** memoria insufficiente ***",
    "*** fine testo ***",
    "*** inizio testo ***",

    "*** Text prea lung ***",
    "*** Nu exista text salvat in memorie ***",
    "*** La inceputul textului ***",
    "*** La sfirsitul textului ***",

    "*** nedostatek pameti ***",
    "*** zadny text neni v pameti ***",
    "*** konec textu ***",
    "*** zacatek textu ***",

#endif
    NULL
};

static void delwrap(VU vu)
{
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE *ln = ctrl->ed_current;
    EDLINE *next;
    int     len, pos;
    int     update = 0;
    char    oldmask;
    char    *p, *q;

    while(ln->ed_next && !(ln->ed_mask & ED_NEWLINE)) {
        len = strlen(ln->ed_data);
        while(len > 0)
            if(ln->ed_data[--len] != ' ')
                break;
        ++len;
        p = ln->ed_next->ed_data;
        pos = opts->ed_width - 1 - len;
        while(*p == ' ')
            ++p;
        if(pos > strlen(p))
            pos = strlen(p);
        else if(pos > 0) {
            while(pos)
                if(p[pos] == ' ')
                    break;
                else
                    --pos;
        }
        if (pos > 0 && ((len + pos + 1) < opts->ed_width)) {
            ++update;
            ln->ed_data[len++] = ' ';
            strncpy(ln->ed_data + len, p, pos);
            ln->ed_data[len + pos] = 0;
            q = &p[pos];
            while(*q == ' ')
                ++q;
            if(!*q) {
                oldmask = ln->ed_next->ed_mask;
                next = ln->ed_next->ed_next;
                ln->ed_next->ed_next = ctrl->ed_free;
                ctrl->ed_free = ln->ed_next;
                ctrl->ed_free->ed_mask = 0;
                ln->ed_next = next;
                if(next)
                    next->ed_prev = ln;
                else
                    ctrl->ed_last = ln;
                if(oldmask & ED_NEWLINE) {
                    ln->ed_mask |= ED_NEWLINE;
                    break;
                }
            }
            else
                strcpy(ln->ed_next->ed_data, q);
            ln=ln->ed_next;
        }
        else
            break;
    }
    if(update) {
        vu->vustatline = (STRING)ctrl->ed_statline;
        vu->vustatcolor = STAT_INFO;
        VUUpdatePad(vu);
    }
}

static void inswrap(VU vu)
{
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE  *ln  = ctrl->ed_current;
    EDLINE *next;
    char   *wrap = ctrl->ed_wrap;
    char    *p, *q, *base;
    int     len;
    char    oldmask;
    int     left = 0;

    if(!opts->ed_wrap)
        return;

    len = strlen(ln->ed_data);
    if(len < opts->ed_width)
        return;

    if(VUGetAttr(vu, HASBORDER))
        ++left;

    base = wrap;
    vu->vustatline = (STRING)ctrl->ed_statline;
    vu->vustatcolor = STAT_INFO;
    for(;;) {
        strcpy(base, ln->ed_data);
        len = strlen(wrap);
        if(len < opts->ed_width) {
            strcpy(ln->ed_data, wrap);
            break;
        }
        len = opts->ed_width;
        while(len > 0) {
            if(wrap[--len] != ' ')
                break;
        }
        q = &wrap[len];
        p = wrap;
        while(*p && *p == ' ')
            ++p;
        while(q > p) {
            if(*q == ' ')
                break;
            --q;
        }
        while(q > p) {
            if(*(q - 1) == ' ')
                --q;
            else
                break;
        }
        if(p < q) {
            *q = 0;
            strcpy(ln->ed_data, wrap);
            while(*(++q) == ' ')
                ;
            strcpy(wrap, q);
            base = wrap + strlen(wrap);
        }
        else {
            strcpy(ln->ed_data, wrap);
            break;
        }
        if((ln->ed_mask & ED_NEWLINE) || !ln->ed_next) {
            oldmask = ln->ed_mask;
            ln->ed_mask &= ~ED_NEWLINE;
            if(!ctrl->ed_free) {
                vu->vustatline = (STRING)MSG_MEM;
                vu->vustatcolor = STAT_ERROR;
                VUTermAlert(1);
                break;
            }
            next = ln->ed_next;
            ln->ed_next = ctrl->ed_free;
            ctrl->ed_free = ctrl->ed_free->ed_next;
            ln->ed_next->ed_next = next;
            ln->ed_next->ed_prev = ln;
            if(next)
                next->ed_prev = ln->ed_next;
            else
                ctrl->ed_last = ln->ed_next;
            ln->ed_next->ed_mask = oldmask;
            strcpy(ln->ed_next->ed_data, wrap);
            break;
        }
        else if(wrap != base)
            *(base++) = ' ';
        ln = ln->ed_next;
    }
    if(vu->vucurcol - left > strlen(ctrl->ed_current->ed_data)) {
        vu->vucurcol = vu->vucurcol - left - strlen(ctrl->ed_current->ed_data);
        ctrl->ed_current = ctrl->ed_current->ed_next;
        ++ctrl->ed_clineno;
    }
    VUUpdatePad(vu);
}

DEFPORT(VUMemoPad)
{
    EDOPTS  *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL  *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE  *ln, *base, *next;
    int     lines;
    int     left = 0;
    int     len, pos;
    int     max;
    char    oldmask;
    char    *p;

    if(VUGetAttr(vu, HASBORDER))
        ++left;

    switch(event) {
    case VU_NOTIFY:
        vu->vustatline = (STRING)ctrl->ed_statline;
        vu->vustatcolor = STAT_INFO;
        break;
    case VU_PAINT:
        if(ctrl->ed_current) {
            vu->vucurrow = ctrl->ed_clineno - opts->ed_lineno;
            if(left || VUGetAttr(vu, HASMENUBAR))
                ++vu->vucurrow;
            len = strlen(ctrl->ed_current->ed_data);
            vu->vucurmode = 1;
            if((vu->vucurcol - left) > len)
                vu->vucurcol = len;
        }
        else
            vu->vucurmode = 0;
        break;
    case VU_KEYBOARD:
        if(vu != front)
            break;

        lines = vu->vudrawn.bt - vu->vudrawn.tp + 1;
        if(VUGetAttr(vu, HASBORDER))
            lines -=2;
        else if(VUGetAttr(vu, HASMENUBAR))
            lines -=1;

        base = (EDLINE *)vu->vupaint;
        ln = ctrl->ed_current;

        switch(p1) {
        case 0x09:
            if(ln->ed_mask & ED_READONLY)
                return TRUE;
            len = strlen(ln->ed_data);
            if(len > (opts->ed_width - 8))
                return TRUE;
            pos = vu->vucurcol - left;
            max = 4 - (pos % 4);
            while(len >= pos) {
                ln->ed_data[len + max] = ln->ed_data[len];
                --len;
            }
            vu->vucurcol += max;
            while(max--)
                ln->ed_data[pos + max] = ' ';
            VUUpdatePad(vu);
            return TRUE;
        case 0x16:
            if(insert_mode)
                insert_mode = FALSE;
            else
                insert_mode = TRUE;
            return TRUE;
        case 0x15:
            if(!ctrl->ed_free)
                return TRUE;
            if(!ctrl->ed_free->ed_mask) {
                VUTermAlert(1);
                vu->vustatcolor = STAT_ERROR;
                vu->vustatline = (STRING)MSG_EMPTY;
                fixcursor();
                return TRUE;
            }
            ctrl->ed_current = ctrl->ed_free;
            ctrl->ed_free = ctrl->ed_free->ed_next;
            ctrl->ed_current->ed_next = ln;
            ctrl->ed_current->ed_prev = ln->ed_prev;
            vu->vustatline = (STRING)ctrl->ed_statline;
            vu->vustatcolor = STAT_INFO;
            if(ln->ed_prev)
                ln->ed_prev->ed_next = ctrl->ed_current;
            else
                ctrl->ed_last = ctrl->ed_current;
            ln->ed_prev = ctrl->ed_current;
            if(base == ln)
                vu->vupaint = (char *)ctrl->ed_current;
            VUUpdatePad(vu);
            return TRUE;
        case 0x04:
        case KEY_DELETE:
            pos = vu->vucurcol - left;
            p = ln->ed_data + pos;
            if(!*p)
                return TRUE;
            strcpy(p, p + 1);
            delwrap(vu);
            VUUpdatePad(vu);
            return TRUE;
        case 0x0B:
        case 0x19:
            if(ln->ed_next) {
                if(ln->ed_prev)
                    ln->ed_prev->ed_next = ln->ed_next;
                else
                    ctrl->ed_first = ln->ed_next;
                if(base == ln)
                    vu->vupaint = (char *)ln->ed_next;
                ln->ed_next->ed_prev = ln->ed_prev;
                ctrl->ed_current = ln->ed_next;
                ln->ed_next = ctrl->ed_free;
                ctrl->ed_free = ln;
            }
            else if(ln->ed_prev) {
                ln->ed_prev->ed_next = NULL;
                ctrl->ed_last = ln->ed_prev;
                --ctrl->ed_clineno;
                if(ln == base) {
                    vu->vupaint = (char *)ln->ed_prev;
                    --opts->ed_lineno;
                }
                ctrl->ed_current = ln->ed_prev;
                ln->ed_next = ctrl->ed_free;
                ctrl->ed_free = ln;
            }
            else {
                ln->ed_data[0] = 0;
                vu->vucurcol = left;
            }
            vu->vustatline = (STRING)ctrl->ed_statline;
            vu->vustatcolor = STAT_INFO;
            VUUpdatePad(vu);
            return TRUE;
        case 0x14:
            if(opts->ed_wrap)
                opts->ed_wrap = FALSE;
            else
                opts->ed_wrap = TRUE;
            return TRUE;
        case 2:
            vu->vucurcol = left;
            fixcursor();
            return TRUE;
        case 5:
            vu->vucurcol = left + strlen(ln->ed_data);
            fixcursor();
            return TRUE;
        case 0x07:
            ln->ed_mask &= ~ED_NEWLINE;
            delwrap(vu);
            return TRUE;
        case 0x17:
            if(!(vu->vucurcol - left))
                return TRUE;
            p = ln->ed_data;
            pos = len = vu->vucurcol - left;
            while(p[len] && p[len] != ' ') {
                ++pos;
                ++len;
                ++vu->vucurcol;
            }
            while(p[len] == ' ') {
                ++len;
                ++vu->vucurcol;
            }
            while(--pos) {
                if(p[pos] != ' ') {
                    while(--pos) {
                        if(p[pos] == ' ')
                            break;
                    }
                    break;
                }
            }
            if(p[pos] == ' ')
                ++pos;
            strcpy(p + pos, p + len);
            vu->vucurcol -= (len - pos);
            delwrap(vu);
            VUUpdatePad(vu);
            return TRUE;
        case KEY_ENTER:
            if(!ctrl->ed_free) {
                VUTermAlert(1);
                vu->vustatline = (STRING)MSG_MEM;
                vu->vustatcolor = STAT_ERROR;
                fixcursor();
                return TRUE;
            }
            next = ln->ed_next;
            ln->ed_next = ctrl->ed_free;
            ctrl->ed_free = ctrl->ed_free->ed_next;
            ln->ed_next->ed_next = next;
            ln->ed_next->ed_prev = ln;
            if(next)
                next->ed_prev = ln->ed_next;
            else
                ctrl->ed_last = ln->ed_next;

            /* Never split RO Lines */

            if(ln->ed_mask & ED_READONLY)
                vu->vucurcol = left + strlen(ln->ed_data);

            strcpy(ln->ed_next->ed_data, &ln->ed_data[vu->vucurcol - left]);
            ln->ed_data[vu->vucurcol - left] = 0;
            oldmask = ln->ed_mask;
            ln->ed_mask |= ED_NEWLINE;
            ln = ln->ed_next;

            /* when inserting into RO area, set pal right and fix NEWLINE */

            if(oldmask & ED_READONLY)
                ln->ed_mask = vu->vupal | ED_NEWLINE;
            else
                ln->ed_mask = oldmask;
            ++ctrl->ed_clineno;
            ctrl->ed_current = ln;
            vu->vucurcol = left;
            if((ctrl->ed_clineno - opts->ed_lineno) >= lines)
                VUScrollPad(vu, lines / 3, FALSE);
            else {
                vu->vustatline = (STRING)ctrl->ed_statline;
                vu->vustatcolor = STAT_INFO;
                VUUpdatePad(vu);
            }
            return TRUE;
        case KEY_HOME:
            ctrl->ed_current = ctrl->ed_first;
            opts->ed_lineno = ctrl->ed_clineno = 1;
            vu->vupaint = (char *)ctrl->ed_first;
            VUUpdatePad(vu);
            return TRUE;
        case KEY_END:
            ctrl->ed_current = ctrl->ed_last;
            vu->vupaint = (char *)ctrl->ed_last;
            opts->ed_lineno = ctrl->ed_clineno = ctrl->ed_llineno;
            VUScrollPad(vu, -(--lines), FALSE);
            return TRUE;
        case KEY_DOWN:
            if(!ln->ed_next) {
                vu->vustatcolor = STAT_ERROR;
                vu->vustatline = (STRING)MSG_EOT;
                fixcursor();
                return TRUE;
            }
            vu->vustatline = (STRING)ctrl->ed_statline;
            vu->vustatcolor = STAT_INFO;
            ctrl->ed_current = ln->ed_next;
            ++ctrl->ed_clineno;
            len = strlen(ctrl->ed_current->ed_data);
            if(vu->vucurcol - left > len)
                vu->vucurcol = len + left;
            if((ctrl->ed_clineno - opts->ed_lineno) >= lines)
                VUScrollPad(vu, lines / 3, FALSE);
            else {
                ++vu->vucurrow;
                fixcursor();
            }
            return TRUE;
        case KEY_UP:
            if(!ln->ed_prev) {
                vu->vustatline = (STRING)MSG_BOT;
                vu->vustatcolor = STAT_ERROR;
                fixcursor();
                return TRUE;
            }
            vu->vustatline = (STRING)ctrl->ed_statline;
            vu->vustatcolor = STAT_INFO;
            ctrl->ed_current = ln->ed_prev;
            --ctrl->ed_clineno;
            len = strlen(ctrl->ed_current->ed_data);
            if(vu->vucurcol - left > len)
                vu->vucurcol = len + left;
            if(ln == base)
                VUScrollPad(vu, - lines / 3, FALSE);
            else {
                --vu->vucurrow;
                fixcursor();
            }
            return TRUE;
        case 0x10:
        case KEY_PAGEUP:
            VUScrollPad(vu, - (--lines), TRUE);
            return TRUE;
        case 0x0e:
        case KEY_PAGEDOWN:
            VUScrollPad(vu, --lines, TRUE);
            return TRUE;
        case KEY_RIGHT:
            if(!ln->ed_data[vu->vucurcol - left]) {
                if(!ln->ed_next) {
                    vu->vustatline = (STRING)MSG_EOT;
                    vu->vustatcolor = STAT_ERROR;
                    fixcursor();
                    return TRUE;
                }
                vu->vustatline = (STRING)ctrl->ed_statline;
                vu->vustatcolor = STAT_INFO;
                ++ctrl->ed_clineno;
                ctrl->ed_current = ln->ed_next;
                vu->vucurcol = left;
                if((ctrl->ed_clineno - opts->ed_lineno) >= lines)
                    VUScrollPad(vu, lines / 3, FALSE);
                else {
                    ++vu->vucurrow;
                    fixcursor();
                }
            }
            else {
                ++vu->vucurcol;
                fixcursor();
            }
            return TRUE;
        case 0x08:
        case 0x7f:
            if(vu->vucurcol > left) {
                max = vu->vucurcol - left;
                len = strlen(ln->ed_data);
                while(max <= len) {
                    ln->ed_data[max - 1] = ln->ed_data[max];
                    ++max;
                }
                ln->ed_data[max] = 0;
                --vu->vucurcol;
                VUUpdatePad(vu);
                delwrap(vu);
                return TRUE;
            }
        case KEY_LEFT:
            if(vu->vucurcol <= left) {
                if(!ln->ed_prev) {
                    vu->vustatline = (STRING)MSG_BOT;
                    vu->vustatcolor = STAT_ERROR;
                    return TRUE;
                }
                vu->vustatline = (STRING)ctrl->ed_statline;
                vu->vustatcolor = STAT_INFO;
                if(ln == base)
                    VUScrollPad(vu, - lines / 3, FALSE);
                ln = ln->ed_prev;
                --ctrl->ed_clineno;
                vu->vucurcol = strlen(ln->ed_data) + left;
                ctrl->ed_current = ln;
                VUUpdatePad(vu);
            }
            else {
                --vu->vucurcol;
                fixcursor();
            }
            if(p1 == 0x08 || p1 == 0x7f)
                delwrap(vu);
            return TRUE;
        default:
            if(p1 > 255)
                break;

            if(!VUValidText(p1))
                break;

            if(ln->ed_mask & ED_READONLY)
                return TRUE;
            max = vu->vucurcol - left;
            len = strlen(ln->ed_data);
            if(len >= opts->ed_width)
                return TRUE;
            if(max >= len) {
                ln->ed_data[len++] = (char)p1;
                ln->ed_data[len] = 0;
            }
            else if(insert_mode) {
                while(len >= max) {
                    ln->ed_data[len + 1] = ln->ed_data[len];
                    --len;
                }
                ln->ed_data[max] = (char)p1;
            }
            else
                ln->ed_data[max] = (char)p1;
            ++vu->vucurcol;
            inswrap(vu);
            if((ctrl->ed_clineno - opts->ed_lineno) >= lines)
                VUScrollPad(vu, lines / 3, FALSE);
            else
                VUUpdatePad(vu);
            return TRUE;
        }
        break;
    case VU_CREATE:
        if(!USEPORT(VUTextPad))
            return FALSE;

        opts->ed_width = vu->vudrawn.rt - vu->vudrawn.lt - (left * 2);

        max = MAXEDIT;

        if(p1 > TRUE) {
            if((unsigned)p1 < MAXEDIT)
                max = p1;
            p1 = TRUE;
        }

        vu->vuimage = malloc(sizeof(EDCTRL) + max);
        ctrl = (EDCTRL *)vu->vuimage;
        ln = (EDLINE *)((char *)vu->vuimage + sizeof(EDCTRL));
        if(!ctrl)
            return FALSE;
        lines = max / (opts->ed_width + 1 + sizeof(EDLINE));
        ctrl->ed_free = ln;
        ctrl->ed_current = NULL;
        ctrl->ed_first = NULL;
        ctrl->ed_last = NULL;
        ctrl->ed_statline = (char *)vu->vustatline;
        opts->ed_wrap = TRUE;

        lines -= 2;
        while(--lines) {
            ln->ed_mask = 0;
            ln->ed_next = (EDLINE *)((char *)ln + opts->ed_width + 1 + sizeof(EDLINE));
            ln = ln->ed_next;
        }
        ln->ed_next = NULL;
        ctrl->ed_wrap = (char *)ln + opts->ed_width + 1 + sizeof(EDLINE);
        VUNewEditText(vu, vu->vupal + 1);
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUTextPad);
}
