/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This code is used to implement basic menu functionality in the VU
 * interface.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/multi.h>

DEFPORT(VUMenuPort)
{
    SELECT  menu    =   (SELECT)vu->vuoptions;
    int     pos     =   menu->sel_mark;
    char    **paint =   (char **)vu->vupaint;
    int     rtn;

    switch(event) {
    case VU_PAINT_SELECT:
    case VU_PAINT:
        vu->vucurcol = 0;
        vu->vucurrow = pos - menu->sel_base;
        if(VUGetAttr(vu, (HASMENUBAR | HASBORDER)))
            ++vu->vucurrow;
        if(VUGetAttr(vu, HASBORDER))
            ++vu->vucurcol;
        vu->vucurmode = 1;
        break;
    case VU_CREATE:
        rtn = USEPORT(VUSelectPort);
        if(rtn) {
            menu->sel_mark = -1;
            VUPostEvent(vu, VU_KEYBOARD, KEY_DOWN, insert_mode);
        }
        return rtn;
    case VU_KEYBOARD:
        switch(p1) {
        case KEY_PAGEUP:
        case KEY_UP:
            while(--pos >= 0) {
                if(paint[pos][0] && paint[pos][0] != '*' && paint[pos][0] != '-') {
                    menu->sel_mark = pos;
                    if(pos < menu->sel_base)
                        menu->sel_base = pos;
                    VUPostRedraw(VU_PAINT_SELECT, vu, NULL);
                    break;
                }
            }
            return TRUE;
        case KEY_PAGEDOWN:
        case KEY_DOWN:
            while(++pos < menu->sel_size) {
                if(paint[pos][0] && paint[pos][0] != '*' && paint[pos][0] != '-') {
                    menu->sel_mark = pos;
                    if(pos >= menu->sel_base + menu->sel_list)
                        menu->sel_base = pos - menu->sel_list + 1;
                    VUPostRedraw(VU_PAINT_SELECT, vu, NULL);
                    break;
                }
            }
            return TRUE;
        case ' ':
        case KEY_ENTER:
            VUPostEvent(NULL, VU_MENU, vu->vuresid, pos + 1);
            break;
        case '?':
        case KEY_F1:
        case KEY_HELP:
            VUPostEvent(NULL, VU_HELP, vu->vuresid, pos + 1);
            return TRUE;
        }
        break;
    case VU_DESTROY:
        VUPostEvent(NULL, VU_MENU, vu->vuresid, 0);
    default:
        break;
    }
    return USEPORT(VUSelectListPort);
}

DEFPORT(VUMenuKeyPort)
{
    SELECT opts = (SELECT)vu->vuoptions;
    char   **list = (char **)vu->vupaint;

    int rtn;

    rtn = USEPORT(VUMenuPort);
    if((event == VU_KEYBOARD) && (rtn == TRUE))
        if(p1 >= 'A' && p1 <= 'z')
            if(VUToLower(p1) == VUToLower(list[opts->sel_mark][0])) {
                VUFlush();
                VUPostEvent(vu, VU_KEYBOARD, KEY_ENTER, 0);
            }
    return rtn;
}
