/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/queue.h>

#define MAX_REGIONS 128

static REGION   *regions = NULL;
static int      region_count = 0;

BOOL VURegisterRegion(VU vu, CITEM *ctrl, EVENT event, PARM p1, PARM p2, RECT *rc)
{
    if(!region_mode)
        return FALSE;

    if(!regions)
        if(NULL == (regions = (REGION *)malloc(MAX_REGIONS * sizeof(REGION))))
            return FALSE;

    if(region_count >= MAX_REGIONS)
        return FALSE;

    regions[region_count].r_vu = vu;
    regions[region_count].r_ctrl = ctrl;
    regions[region_count].r_event = event;
    regions[region_count].r_p1 = p1;
    regions[region_count].r_p2 = p2;

    if((p1 == -1) && (p2 == -1))
        regions[region_count].r_rpos = TRUE;
    else
        regions[region_count].r_rpos = FALSE;

    memcpy(&regions[region_count].r_region, rc, sizeof(RECT));
    ++region_count;
    return TRUE;
}

int VUGetRegion(int row, int col)
{
    int i = 0;
    RECT *rc;

    while(i < region_count) {
        rc = &regions[i].r_region;
        if(row < rc->tp || row > rc->bt)
            continue;
        if(col < rc->lt || col > rc->rt)
            continue;

        if(regions[i].r_rpos) {
            regions[i].r_p1 = row - rc->tp;
            regions[i].r_p2 = col - rc->lt;
        }
        return i + 1;
    }
    return 0;
}

void VUPostRegion(int id)
{
    REGION  *r;
    CITEM   *control;
    VU  vu;

    if(id < 1 || id > region_count)
        return;

    r = &regions[--id];
    vu = r->r_vu;
    control = r->r_ctrl;
    if(vu != front) {
        if(control)
            vu->vucontrol = control;
        VUToFront(vu);
    }
    else if(control) {
        if(vu->vucontrol != control) {
            vu->vucontrol = control;
            (*(vu->vuproc))(vu, VU_PAINT, 0, 0);
        }
    }
    VUPostEvent(vu, r->r_event, r->r_p1, r->r_p2);
}

