/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Message box procedure.   Operates as a locked window object
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>

DEFPORT(VUMsgboxPort)
{
    STRING  p;
    int     row,col;
    int     rtn;
    int     indent = 0;
    char    *space = NULL;
    CELL    cell;
    CELL    base;
    RECT    rc;

    memcpy(&rc,&vu->vudrawn,sizeof(rc));

    switch(event) {
    case VU_PAINT:
        rtn = VUWindowPort(vu, event, p1, 0);
        cell = ((vu->vupal << 8) + 0x0100);
        base = (vu->vupal << 8);
        row = rc.tp + 2;
        col= rc.lt + 2;
        p = VUTextString(vu->vupaint);
        while(*p) {
            if(*p == '\\') {
                switch(*(++p)) {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                    cell = (base + (*p - '0')) << 8;
                    break;
                case 't':
                    col += (4 - col % 4);
                    break;
                case 'n':
                    *p = '\n';
                    break;
                case 'i':
                    indent = col - rc.lt - 2;
                    break;
                case 'u':
                    indent = 0;
                }
                if(!*p)
                    break;
                if(*p != '\\' && *p != '\n') {
                    ++p;
                    continue;
                }
            }
            if(row > (rc.bt - 1))
                break;
            if(*p=='\n') {
                ++row;
                space = NULL;
                col = rc.lt + 2 + indent;
            }
            else if(col <= (rc.rt - 2)) {
                if(*p == ' ')
                    space = (char *)p;
                VUDrawCell((CELL)*p | cell,row,col++);
            }
            else {
                if(space) {
                    while(*p != ' ') {
                        --p;
                        VUDrawCell(base | 0x0020 ,row,--col);
                    }
                }
                else
                    --p;
                space = NULL;
                ++row;
                col = rc.lt + indent + 2;
            }
            ++p;
        }
        return rtn;
    default:
        break;
    }
    return VUWindowPort(vu,event,p1,p2);
}

VU VUYesNoBox(VU parent, STRING str)
{
    int l, w, i, offset = 3;

    VUTextSize((char *)str, &l, &w);

    if(w < 20)
        w=20;
    else
        offset+=(w-20)/2;
    YesNoBox[0].cwid=w + 4;
    YesNoBox[0].clen=l + 6;
    for(i = 2; i < (NR_COUNT * 3); i+= 3)
    {
        YesNoBox[i].ccol = offset;
        YesNoBox[i + 1].ccol = offset + 10;
        YesNoBox[i].crow = l + 3;
        YesNoBox[i + 1].crow = l + 3;
    }
    return VUCreate(YesNoBox, NULL, -1, -1, VUNULL, 0xffff, parent, str);
}

DEFWINDOW(YesNoBox)
    WIN_CLASS(VUMsgboxPort,NULL)
    WIN_ATTR(5,20,HASSHADOW | HASBORDER | LOCKED,32,NULL)

    REPLACE_ENGLISH(NULL, NULL)
    BUTTON_CONTROL((STRING)"  ~Yes  ",    0,      3,  7,  VU_ACCEPT)
    BUTTON_CONTROL((STRING)"  ~No   ",    0,      3,  7,  VU_CANCEL)

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, NULL)
    BUTTON_CONTROL((STRING)"  ~Si  ",    0,      3,  6, VU_ACCEPT)
        BUTTON_CONTROL((STRING)"  ~No  ",    0,      3,  6, VU_CANCEL)
        REPLACE_ROMANIAN(NULL, NULL)
        BUTTON_CONTROL((STRING)"  ~Da  ",    0,      3,  6, VU_ACCEPT)
        BUTTON_CONTROL((STRING)"  ~Nu  ",    0,      3,  6, VU_CANCEL)
        REPLACE_CZECH(NULL, NULL)
        BUTTON_CONTROL((STRING)"  ~Ano  ",    0,      3,  7, VU_ACCEPT)
        BUTTON_CONTROL((STRING)"  ~Ne   ",     0,      3,  7, VU_CANCEL)
#endif
ENDDEF

