/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>

enum
{
    ORDER_MDY = 0,
    ORDER_DMY,
    ORDER_YMD
};

#define DAY (60l * 60l * 24l)
#define hour12(x)   ((x % 12) == 0 ? 12 : x % 12)
#define ampm(x)     (x > 11 ? "pm" : "am")

CONST   char    datesep[] = {'/', '/', '.'};
CONST   char    timesep[] = {':', ':', '.', ' '};
CONST   char    clk24[] =   { 0,   1,   1,   1};
CONST   int     dateorder[] = {ORDER_MDY, ORDER_DMY, ORDER_DMY};

static  char    datestr[65];

char *VUDateDMY(time_t *time)
{
    struct  tm  *dt = VULocalTime(time);

    sprintf(datestr, "%2d %s %d",
         dt->tm_mday, VUMultiMonth(dt->tm_mon, 0), VUMultiCentury(time));
    return datestr;
}

char *VUDateMDY(time_t *time)
{
    struct  tm  *dt = VULocalTime(time);

    sprintf(datestr, "%s %d, %d",
        VUMultiMonth(dt->tm_mon, 0), dt->tm_mday, VUMultiCentury(time));
    return datestr;
}

char *VUShortDate(time_t *time)
{
    struct  tm  *dt = VULocalTime(time);

    switch(dateorder[vu_national_date]) {
    case ORDER_MDY:
        sprintf(datestr, "%s %d, %d",
            VUMultiMonth(dt->tm_mon, 1), dt->tm_mday, VUMultiCentury(time));
        return datestr;
    case ORDER_DMY:
    default:
        sprintf(datestr, "%d %s %d",
            dt->tm_mday, VUMultiMonth(dt->tm_mon, 1), VUMultiCentury(time));
        return datestr;
    }
}

char *VULongDate(time_t *time)
{
    switch(dateorder[vu_national_date]) {
    case ORDER_MDY:
        return VUDateMDY(time);
    case ORDER_DMY:
    default:
        return VUDateDMY(time);
    }
}

char *VUUnixDate(time_t *time)
{
    struct tm *dt = VULocalTime(time);
    char tzone[12];
    char tsep = timesep[vu_national_time];

#ifdef  TZSET
    tzset();
    tzone[0] = ' ';
    if(dt->tm_isdst)
        strcpy(&tzone[1], tzname[1]);
    else
        strcpy(&tzone[1], tzname[0]);
    strcat(tzone, " ");
#else
    strcpy(tzone, " ");
#endif

    switch(dateorder[vu_national_date]) {
    case ORDER_MDY:
        sprintf(datestr, "%s %s %2d %02d%c%02d%c%02d%s%d",
        VUMultiDow(dt->tm_wday, 1), VUMultiMonth(dt->tm_mon, 1), dt->tm_mday,
        dt->tm_hour, tsep, dt->tm_min, tsep, dt->tm_sec, tzone, VUMultiCentury(time));
        break;

    case ORDER_DMY:
        sprintf(datestr, "%s %2d %s %02d%c%02d%c%02d%s%d",
        VUMultiDow(dt->tm_wday, 1), dt->tm_mday, VUMultiMonth(dt->tm_mon, 1),
        dt->tm_hour, tsep, dt->tm_min, tsep, dt->tm_sec, tzone, VUMultiCentury(time));
        break;
    }
    return datestr;
}

char *VUMixedDate(time_t *date)
{
    struct tm *dt = VULocalTime(date);
    time_t now;
    char tsep = timesep[vu_national_time];

    time(&now);

    if((*date < now + DAY) && (*date > (now - 5l * DAY))) {
        if(clk24[vu_national_time]) {
            sprintf(datestr, "%s %2d%c%02d",
                VUMultiDow(dt->tm_wday, 1), dt->tm_hour, tsep, dt->tm_min);
        }
        else
            sprintf(datestr, "%s %2d%c%02d%s",
                VUMultiDow(dt->tm_wday, 1), hour12(dt->tm_hour), tsep, dt->tm_min, ampm(dt->tm_hour));
    }
    else {
        if(vu_national_abbrv)
            return VUCvtDate(dt);
        switch(dateorder[vu_national_date]) {
        case ORDER_MDY:
            sprintf(datestr, "%s %02d %04d",
                VUMultiMonth(dt->tm_mon, 1), dt->tm_mday, dt->tm_year + 1900);
            break;
        case ORDER_DMY:
            sprintf(datestr, "%02d %s %04d",
                dt->tm_mday, VUMultiMonth(dt->tm_mon, 1), dt->tm_year + 1900);
            break;
        }
    }
    if(datestr[0] > 95)
        datestr[0] -= 32;
    return datestr;
}

