/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    This module allows you to test varios screen locations and
    regions to determine if they have been obscured by an object
    in front of you.  These services may be used when updating
    a window that is not the front to determine if direct access
    to the imageport memory is possible, or if alternate (slower)
    updating with checking for each cell will be nessisary.
*/

#include <vu-config.h>
#include <worldvu/vu.h>

BOOL VUObscured(VU vu)
{
    /*  test to see if the rectangular region of another window
        is obscuring this one */

    VU      vp = front;
    RECT    *r1, *r2;

    r1 = &vu->vudrawn;

    while(vp) {
        if(vp == vu)
            return FALSE;

        r2=&vp->vudrawn;
        if((r1->tp > r2->bt) ||
           (r1->bt < r2->tp) ||
           (r1->lt > r2->rt) ||
           (r1->rt < r2->lt)) {
            vp=vp->vunext;
        }
        else
            return TRUE;
    }
    return FALSE;
}

BOOL VUCellObscured(VU vu, int row, int col)
{
    RECT    *rc;
    VU      vp = front;

    while(vp) {
        if(vp == vu)
            return FALSE;

        rc = &vp->vudrawn;
        if((row >= rc->tp) &&
           (row <= rc->bt) &&
           (col >= rc->lt) &&
           (col <= rc->rt))
            return TRUE;

        vp = vp->vunext;
    }
    return FALSE;
}
