/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <ctype.h>

void VUScrollClear(VU vu)
{
    SELECT  opts = (SELECT)vu->vuoptions;
    STRING  *list = (STRING *)vu->vupaint;
    int     i;

    for(i = 0; i <opts->sel_list; ++i)
        *list[i] = 0;

    if(vu == front)
        VUPostRedraw(VU_PAINT_SELECT, vu, NULL);
    else
        VUPostRedraw(VU_PAINT, NULL, &vu->vudrawn);
}

void VUScrollText(VU vu, char *text)
{
    SELECT  opts = (SELECT)vu->vuoptions;
    STRING  *list = (STRING *)vu->vupaint;
    int i = 0;

    while(list[i]) {
        list[i] = list[i + 1];
        ++i;
    }
    strcpy((char *)list[--i], text);

    if(opts->sel_mark)
        return;
    ++opts->sel_mark;

    if(vu == front)
        VUPostRedraw(VU_PAINT_SELECT, vu, NULL);
    else
        VUPostRedraw(VU_PAINT, NULL, &vu->vudrawn);
}

DEFPORT(VUScrollPort)
{
    register SELECT opts = (SELECT)vu->vuoptions;
    int         top, left, width = 0;
    STRING      *list = (STRING *)vu->vupaint;
    STRING      ln;
    PAL         pal;
    int         rtn, i;
    CELL        arrow;

    switch(event) {
    case VU_KEYBOARD:
        return USEPORT(VUWindowPort);
    case VU_DESTROY:
        free(vu->vupaint);
        return TRUE;
    case VU_CREATE:
        vu->vupaint = NULL;
        rtn = USEPORT(VUSelectPort);
        if(!rtn)
            return FALSE;
        list = (STRING *)malloc(sizeof(char *) * (opts->sel_list + 1));
        ln = (STRING)malloc(opts->sel_list * (width + 8));
        vu->vuimage = (STRING)ln;
        list[opts->sel_list] = NULL;
        for(i = 0; i <opts->sel_list; ++i) {
            list[i] = ln + (i * (width + 8));
            *list[i] = 0;
        }
        vu->vupaint = (char *)list;
        return TRUE;

    case VU_PAINT:
        USEPORT(VUWindowPort);

    case VU_PAINT_SELECT:
        opts->sel_mark = 0;
        if(opts->sel_box == NULL) {
            top = vu->vudrawn.tp + 1;
            width = vu->vudrawn.rt - vu->vudrawn.lt - 1;
            left = vu->vudrawn.lt + 1;
            if(VUGetAttr(vu, HASBORDER)) {
                arrow = (vu->vupal + 6) << 8 | GRAPHIC_CELL;
                if(speed > 12000 && vu != front)
                    arrow += 0x0100;
            }
            else
                arrow = 0;
        }
        else {
            top = opts->sel_box->crow + vu->vudrawn.tp + 1;
            width = opts->sel_box->cwid - 2;
            left = opts->sel_box->ccol + vu->vudrawn.lt + 1;
        }

        for(i = 0;i < opts->sel_list; ++i) {
            ln = list[i];
            pal = vu->vupal;
            if(*ln && (*ln < 8)) {
                pal += (*ln);
                ++ln;
            }
            VUDrawString(ln, top + i, left + 2, width - 2, pal);
        }
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUWindowPort);
}

