/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    Select list control

    The parent control will create sub-window which will then have
    a list of all data values for the control.  The sub-window will
    access the parent process's private data area.  This is a seldom
    used method for a parent window to share private data with it's
    children.
*/

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <ctype.h>

/*  Private select stuff */

FORWARD PORT(SelectPort);

static  DEFPORT(SelectPort)
{
    CITEM   *control = vu->vuparent->vucontrol;
    SELECT  opts = (SELECT)vu->vuoptions;
    short   *selector = (short *)(control->ctext);
    int     rtn;

    switch(event) {
    case VU_KEYBOARD:
        switch(p1) {
        case KEY_LEFT:
            p1 = KEY_UP;
            break;
        case KEY_RIGHT:
            p1 = KEY_DOWN;
            break;
        case KEY_EXIT:
            opts->sel_mark = *selector;
        case ' ':
        case KEY_ENTER:
            p1 = KEY_EXIT;
        }
        break;
    case VU_CREATE:
        ++control->cstate;
        vu->vupaint = control->cmsg;
        vu->vupal = vu->vuparent->vupal;    /* get parent color */
        rtn = USEPORT(VUSelectListPort);
        opts->sel_mark = *selector;
        while(opts->sel_mark >= (opts->sel_base + opts->sel_list))
            ++opts->sel_base;
        return rtn;
    case VU_DESTROY:
        --control->cstate;
        *selector = opts->sel_mark;
    default:
        break;
    }
    return USEPORT(VUSelectListPort);
}

static DEFWINDOW(Select)
    WIN_CLASS(SelectPort, (STRING)"")
    WIN_ATTR(1, 1, HASBORDER | HASSHADOW, 16, NULL)
ENDDEF

/*  Our Select List Control */

DEFCONTROL(VUSelectListControl)
{
    int     row = control->crow + vu->vudrawn.tp;
    int     col = control->ccol + vu->vudrawn.lt;
    VU      child;
    int     rtn;

    switch(event) {
    case VU_KEYBOARD:

        /*  since our window may be marked as 'ACTIVE', we do not want
            to create multiple select list windows if one has already
            been created */

        if(control->cstate)
            return FALSE;

        switch(p1) {
        case KEY_LEFT:
        case KEY_RIGHT:
            rtn = USECONTROL(VUSelectControl);
            if(!rtn)
                return FALSE;

            if(vu->vucontrol != control)
                return TRUE;

        case ' ':
        case KEY_ENTER:
            if(vu->vucontrol != control)
                break;

            Select[0].cwid = control->cwid + 2;
            Select[0].clen = control->clen + 2;
            child = VUCreateQ(Select, NULL, row - viewport.tp - 1, col - viewport.lt - 1, NULL);
            child->vuparent = vu;
            return TRUE;
        }
    default:
        break;
    }
    return USECONTROL(VUSelectControl);
}

