/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Select list control
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <ctype.h>

DEFCONTROL(VUSelectControl)
{
    char    **clist = (char **)control->cmsg;
    short   *selector = (short *)control->ctext;
    int     width = control->cwid;
    int     row = control->crow + vu->vudrawn.tp;
    int     col = control->ccol + vu->vudrawn.lt;
    BOOL    active = FALSE;
    CELL    cell;

    if(vu->vucontrol == control)
        active = TRUE;

    cell = (CELL)((vu->vupal + 3));

    switch(event) {
    case VU_CREATE:
        return TRUE;
    case VU_ACCEPT:         /* form accept event */
        return TRUE;
    case VU_KEYBOARD:
        switch(p1) {
        case KEY_HOME:
            *selector = 0;
            VUDrawString((STRING)clist[0], row, col, width, cell);
            ++redraw;
            return TRUE;
        case KEY_END:
            while(clist[*selector])
                ++(*selector);
            --(*selector);
            VUDrawString((STRING)clist[*selector], row, col, width, cell);
            ++redraw;
            return TRUE;
        case KEY_ENTER:
        case KEY_DOWN:
        case KEY_UP:
            if(active)
                return FALSE;
        case KEY_LEFT:
            if(active) {
                if(*selector)
                    --(*selector);
                VUDrawString((STRING)clist[*selector], row, col, width, cell);
                ++redraw;
                return TRUE;
            }
        case 0x08:
        case ' ':
        case KEY_RIGHT:
            if(active) {
                ++(*selector);
                if(!clist[*selector]) {
                    if(p1 == ' ')
                        *selector=0;
                    else
                        --(*selector);
                }
                VUDrawString((STRING)clist[*selector], row, col, width, cell);
                ++redraw;
                return TRUE;
            }
        case KEY_PAGEUP:
        case KEY_PAGEDOWN:
        case KEY_TAB:
            if(active)
                return FALSE;
            VUControlCursor(vu, control, 0, 0);
            vu->vucontrol = control;
            return TRUE;
        default:
            return FALSE;
        }

    case VU_PAINT:
        if(VUGetAttr(vu,EDITBAR) && !active)
            cell = (CELL)(vu->vupal);
        VUDrawString((STRING)clist[*selector], row, col, width, cell);
        if(active)
            VUControlCursor(vu, control, 0, 0);

        ++redraw;
        return TRUE;
    case VU_ENTER:
    case VU_EXIT:
        if(VUGetAttr(vu,EDITBAR))
            (*(control->cproc))(vu,control,VU_PAINT,0,0);
        break;
    default:
        break;
    }
    return TRUE;
}

