/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Shell setup.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/queue.h>
#include <worldvu/multi.h>

int vuoptind = 1;
int vuoptopt;
char *vuoptarg;

static  DEFWINDOW(MsgBox)
    WIN_CLASS(VUMsgboxPort, NULL)
    WIN_ATTR(5, 20, HASSHADOW | FAILURE | HASBORDER | LOCKED, 32, (STRING)"Press <ESCAPE> to continue")

#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN(NULL, (STRING)"Premi <ESCAPE> per continuare")
    REPLACE_ROMANIAN(NULL, (STRING)"Apasati <ESCAPE> pentru continuare")
    REPLACE_CZECH(NULL, (STRING)"Chcete-li pokracovat, stisknete <ESC>")
#endif
ENDDEF

void VUError(char *title, char *msg)
{
    int l, w;
    VUTextSize(msg, &l, &w);
    if(w < 20)
        w = 20;

    MsgBox[0].cwid = w + 4;
    MsgBox[0].clen = l + 4;
    VUStartup();
    VUCreateQ(VUDesktop, NULL, -1, -1, NULL);
    VUCreateQ(MsgBox,(STRING)title, -1, -1, msg);
    while(VUDispatch())
        ;
    VUShutdown();
    exit(-1);
}

int VUGetOpt(int argc, char **argv, char *opts, char *usage, char *help)
{
    static int sp = 1;
    int len;
    register int c;
    register char *cp = NULL;
    char title[32];

    if(!argc) {
        if(NULL == (cp = strchr(argv[0],'/')))
            cp = argv[0];
        strcpy(title, cp);
        VUError(title, help);
    }

    if(sp == 1) {
        if(vuoptind >= argc || argv[vuoptind][0] != '-' || argv[vuoptind][1] == '\0')
            return(EOF);
        else if(!strcmp(argv[vuoptind], "--")) {
            vuoptind++;
            return(EOF);
        }
    }
    vuoptopt = c = (int)argv[vuoptind][sp];
    if(c == ':' || (cp=strchr(opts, c)) == NULL) {
        if(c == '?') {
            if(NULL == (cp = strchr(argv[0],'/')))
                cp = argv[0];
            strcpy(title, cp);
            VUError(title, help);
        }
        strcpy(title, "Illegal Option -- ");
        len = strlen(title);
        title[len] = (char)c;
        title[len + 1] = 0;
        VUError(title, usage);
    }
    if(cp && *++cp == ':') {
        if(argv[vuoptind][sp+1] != '\0')
            vuoptarg = &argv[vuoptind++][sp+1];
        else if(++vuoptind >= argc) {
            strcpy(title,"Option Requires Argument -- ");
            len = strlen(title);
            title[len] = (char)c;
            title[len + 1] = 0;
            VUError(title, usage);
        } else
            vuoptarg = argv[vuoptind++];
        sp = 1;
    }
    else {
        if(argv[vuoptind][++sp] == '\0') {
            sp = 1;
            vuoptind++;
        }
        vuoptarg = NULL;
    }
    return(c);
}

