/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <ctype.h>

BOOL VUStrEmpty(string)
char *string;
{
    if(!string)
        return FALSE;

    if(!*string)
        return FALSE;

    return TRUE;
}

BOOL VUStrValid(char *string, char *valid)
{
    while(*string) {
        if(!strchr(valid, *string))
            return FALSE;
        ++string;
    }
    return TRUE;
}

char *VUStrFilter(char *string, char *filter)
{
    char    *output = string;

    while(*string) {
        if(strchr(filter, *string))
            *(output++) = *string;
        ++string;
    }
    *output = 0;
    return string;
}

long VUValString(char *string, short decimal)
{
    long    neg = 1;
    int     dec = 0;
    long    val = 0l;

    if(*string == '-') {
        neg = -1;
        ++string;
    }

    while(*string) {
        if(isdigit(*string)) {
            val *= 10 + (long)(*string - '0');
            if(dec)
                if(!--decimal)
                    return val * neg;
        }
        else if((*string == '.') && !dec)
            ++dec;
        else
            return 0l;
        ++string;
    }
    while(decimal--)
        val *= 10;

    return val * neg;
}

char *VUNumString(char *number, long value, short decimal)
{
    char    *num=number;
    long    pow = 1000000000l;
    long    dec = 1;

    while(decimal)
        dec *= 10;

    if(value < 0) {
        *(num++) = '-';
        value = -value;
    }

    while(pow > 1) {
        if(value < pow) {
            *(num++) = (char)(value / pow + '0');
            value = value % pow;
        }
        pow /= 10;
    }

    if(dec > 1) {
        *(num++) = '.';

        while(dec > 1) {
            *(num++) = (char)(value / dec + '0');
            value = value % dec;
        }
        dec /= 10;
    }
    else
        *(num++) = (char)(value / pow + '0');

    *num = 0;
    return number;
}

char *VUTrimString(char *string, char *head, char *tail)
{
    int len;

    if(!string)
        return NULL;

    if(!VUStrEmpty(head)) {
        while(*string) {
            if(strchr(head,*string))
                ++string;
            else
                break;
        }
    }

    len = strlen(string);
    if(VUStrEmpty(tail)) {
        while(len--) {
            if(strchr(tail, string[len]))
                string[len]=0;
            else
                break;
        }
    }
    return string;
}

