/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * The tag port procedure adds support for a 'tag' button in select lists.
 * This 'tag' button can either be an external button which returns a
 * button 'VU_TAG' event or the space bar key.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>

DEFPORT(VUTagSelectPort)
{
    register    SELECT  opts = (SELECT) vu->vuoptions;
    int     tag = 0;
    char    *p;
    STRING  *list=(STRING *)vu->vupaint;

    switch(event) {
    case VU_KEYBOARD:
        if(p1 == ' ' || p1 == KEY_ENTER) {
            VUEvent(vu,VU_TAG,0,0);
            return TRUE;
        }
        break;
    case VU_BUTTON:
        if(p1 == VU_TAG) {
            VUEvent(vu,VU_TAG,0,0);
            return TRUE;
        }
        break;
    case VU_TAG:
        if(opts->sel_mark >= opts->sel_size)
            return tag;

        VUPostRedraw(VU_PAINT_SELECT,vu,NULL);
        p=(char *)list[opts->sel_mark];
        if(*p == ' ') {
            *p = '*';
            tag = TRUE;
        }
        else {
            *p = ' ';
            tag = FALSE;
        }
        return tag;
    default:
        break;
    }

    return  USEPORT(VUSelectPort);
}
