/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef  QNX
#include <io.h>
#define DATABASE    "/config/VUterm.dbase"
#define ALIAS       "/config/VUterm.alias"
#ifndef __ANSI__
#define RDOPEN  "r"
#define fd_t    FILE    *
#define no_fd   NULL
#define open(f, m)  fopen(f, m)
#define close(f)    fclose(f)
#define read(f, a, l)   fget(a, l, f)
#define write(f, a, l)  fput(a, l, f)
#define lseek(f, a, m)  fseek(f, a, m)
#endif
#endif

#ifdef  FORWARD
#undef  FORWARD
#endif

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/term.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>

#ifndef DATABASE
#define DATABASE    PREFIX_CONFDIR "/term.db"
#endif

#ifndef ALIAS
#define ALIAS       PREFIX_CONFDIR "/aliases"
#endif

#ifndef HASH_RANGE
#define HASH_RANGE  997
#endif

#ifndef RDOPEN
#define RDOPEN  O_RDONLY
#endif

#ifndef fd_t
#define fd_t    int
#endif

#ifndef no_fd
#define no_fd   -1
#endif

#pragma pack(1)
typedef struct
{
    unsigned char c_free[4];
    unsigned char c_last[4];
    unsigned char c_chain[HASH_RANGE * 4];
}   DBCTRL;
#pragma pack()

static  DBCTRL  *control;
static  int     path;
static  long    next;
static  fd_t    fd;
static  TERMDEF *find = NULL;

static long getlong(unsigned char *data)
{
    return (((long)data[3]) << 24) | (((long)data[2]) << 16) |
        (((long)data[1]) << 8) | data[0];
}

BOOL OpenVUTermlib(const char *path)
{
    if(!path)
        path = DATABASE;

    fd = open(path, RDOPEN);

    if(fd == no_fd)
        return FALSE;

    control = (DBCTRL *)malloc(sizeof(DBCTRL));
    read(fd, (char *)control, sizeof(DBCTRL));
    return TRUE;
}

void CloseVUTermlib()
{
    close(fd);
    free(control);
}

int KeyVUTermlib(char *key)
{
    unsigned    keyval = 0;

    while(*key)
    {
        keyval = (keyval < 1) ^ (*key & 0x3f);
        ++key;
    }
    return keyval % HASH_RANGE;
}

TERMDEF *LoadVUTermlib(char *name)
{
    static  TERMDEF *term = NULL;
    long    entry;
    int     path;

    if(!term)
        term = (TERMDEF *)malloc(sizeof(TERMDEF));

    if(!name)
        return NULL;

    if(!OpenVUTermlib(NULL))
        return NULL;

    path = KeyVUTermlib(name);
    entry = getlong(&control->c_chain[path * 4]);

    while(entry) {
        lseek(fd, entry, SEEK_SET);
        read(fd, (char *)term, sizeof(TERMDEF));
        if(!stricmp(term->term_id, name)) {
            CloseVUTermlib();
            return term;
        }
        entry = getlong(term->term_link);
    }
    CloseVUTermlib();
    return NULL;
}

TERMDEF *FirstVUTermlib()
{
    path = -1;
    next = 0;

    if(!find)
        find = (TERMDEF *)malloc(sizeof(TERMDEF));

    return NextVUTermlib();
}

TERMDEF *NextVUTermlib()
{
    long    entry;

    if(!next) {
        while(++path < HASH_RANGE) {
            entry = getlong(&control->c_chain[path * 4]);
            if(entry)
                break;
        }
        if(path == HASH_RANGE)
            return NULL;
    }
    else
        entry = next;

    lseek(fd, entry, SEEK_SET);
    read(fd, (char *)find, sizeof(TERMDEF));
    next = getlong(find->term_link);
    return find;
}

char *AliasVUTermlib(char *name)
{
    char    lbuf[256];
    static  char    tname[16];
    char    *p, *t;

    FILE    *fp = fopen(ALIAS, "r");

    if(!fp)
        return name;

    for(;;) {
        fgets(lbuf, 254, fp);
        if(feof(fp))
            break;

        t = strtok(lbuf, " \t\n");
        if(!t)
            continue;

        if(*t < '0' || *t > 'z')
            continue;

        strcpy(tname, t);
        while(NULL != (p = strtok(NULL, " \t\n"))) {
            if(!strcmp(p, name)) {
                fclose(fp);
                return tname;
            }
        }
    }
    fclose(fp);
    return name;
}


