/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef  QNX
#include <io.h>
#define DATABASE    "/config/VUterm.dbase"
#ifndef __ANSI__
#define RWOPEN  "rw"
#define RWMAKE  "wr"
#define fd_t    FILE    *
#define no_fd   NULL
#define open(f, m)  fopen(f, m)
#define close(f)    fclose(f)
#define read(f, a, l)   fget(a, l, f)
#define write(f, a, l)  fput(a, l, f) ; fflush(f)
#define lseek(f, a, m)  fseek(f, a, m)
#endif
#endif

#ifdef  FORWARD
#undef  FORWARD
#endif
#include <vu-config.h>
#include <fcntl.h>
#include <unistd.h>
#include <worldvu/vu.h>
#include <worldvu/term.h>

#ifndef DATABASE
#define DATABASE    PREFIX_CONFDIR "/term.db"
#endif

#ifndef HASH_RANGE
#define HASH_RANGE  997
#endif

#ifndef RWMAKE
#define RWMAKE  O_RDWR | O_CREAT
#endif

#ifndef RWOPEN
#define RWOPEN  O_RDWR
#endif

#ifndef fd_t
#define fd_t    int
#endif

#ifndef no_fd
#define no_fd   -1
#endif

#pragma pack(1)
typedef struct {
    unsigned char c_free[4];
    unsigned char c_last[4];
    unsigned char c_chain[HASH_RANGE * 4];
}   DBCTRL;
#pragma pack()

static BOOL OpenRWTermlib(void);
static void CloseRWTermlib(void);

static  DBCTRL  *control;
static  fd_t    fd;
static  TERMDEF term;

static long getlong(unsigned char *data)
{
    return (((long)data[3]) << 24) | (((long)data[2]) << 16) |
        (((long)data[1]) << 8) | data[0];
}

static void setlong(unsigned char *data, long value)
{
    data[3] = (value >> 24) & 0xff;
    data[2] = (value >> 16) & 0xff;
    data[1] = (value >> 8) & 0xff;
    data[0] = value & 0xff;
}

static  BOOL    OpenRWTermlib()
{
    int i;

    fd = open(DATABASE, RWOPEN);
    if(fd == no_fd) {
        fd = open(DATABASE, RWMAKE, 0750);
        if(fd == no_fd)
            return FALSE;

        control = (DBCTRL *)malloc(sizeof(DBCTRL));
        setlong(control->c_free, 0);
        setlong(control->c_last, sizeof(DBCTRL));
        for(i = 0; i <HASH_RANGE; ++i)
            setlong(&control->c_chain[i * 4], 0);
        write(fd, (char *)control, sizeof(DBCTRL));
    }
    else {
        control = (DBCTRL *)malloc(sizeof(DBCTRL));
        setlong(control->c_free, 0);
        setlong(control->c_last, sizeof(DBCTRL));
        for(i = 0; i <HASH_RANGE; ++i)
            setlong(&control->c_chain[i * 4], 0);
        read(fd, (char *)control, sizeof(DBCTRL));
        lseek(fd, 0l, SEEK_SET);
        write(fd, (char *)control, sizeof(DBCTRL));
    }
    return TRUE;
}

static  void CloseRWTermlib()
{
    lseek(fd, 0l, SEEK_SET);
    write(fd, (char *)control, sizeof(DBCTRL));
    close(fd);
    free(control);
}

TERMDEF *SaveVUTermlib(TERMDEF *def)
{
    long    entry;
    int     path;

    if(!def)
        return NULL;

    if(!OpenRWTermlib())
        return NULL;

    path = KeyVUTermlib(def->term_id);
    entry = getlong(&control->c_chain[path * 4]);

    while(entry) {
        lseek(fd, entry, SEEK_SET);
        read(fd, (char *)&term, sizeof(term));
        if(!stricmp(term.term_id, def->term_id)) {
            lseek(fd, entry, SEEK_SET);
            write(fd, (char *)def, sizeof(term));
            CloseRWTermlib();
            return &term;
        }
        entry = getlong(term.term_link);
    }
    if(0 != (entry = getlong(control->c_free))) {
        lseek(fd, entry, SEEK_SET);
        read(fd, (char *)&term, sizeof(term));
        setlong(control->c_free, getlong(term.term_link));
    }
    else {
        entry = getlong(control->c_last);
        setlong(control->c_last, entry + sizeof(term));
    }
    setlong(def->term_link, getlong(&control->c_chain[path * 4]));
    setlong(&control->c_chain[path * 4], entry);
    lseek(fd, entry, SEEK_SET);
    write(fd, (char *)def, sizeof(term));

    CloseRWTermlib();
    return def;
}

TERMDEF *DeleteVUTermlib(char *name)
{
    long    entry;
    long    prev = 0;
    long    next;
    int     path;

    if(!name)
        return NULL;

    if(!OpenRWTermlib())
        return NULL;

    path = KeyVUTermlib(name);
    entry = getlong(&control->c_chain[path * 4]);

    while(entry) {
        lseek(fd, entry, SEEK_SET);
        read(fd, (char *)&term, sizeof(term));
        if(!stricmp(term.term_id, name)) {
            next = getlong(term.term_link);
            setlong(term.term_link, getlong(control->c_free));
            setlong(control->c_free, entry);
            lseek(fd, entry, SEEK_SET);
            write(fd, (char *)&term, sizeof(term));
            if(prev) {
                lseek(fd, prev, SEEK_SET);
                read(fd, (char *)&term, sizeof(term));
                setlong(term.term_link, next);
                lseek(fd, prev, SEEK_SET);
                write(fd, (char *)&term, sizeof(term));
            }
            else
                setlong(&control->c_chain[path * 4], next);
            CloseRWTermlib();
            return &term;
        }
        prev = entry;
        entry = getlong(term.term_link);
    }
    CloseRWTermlib();
    return NULL;
}



