/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/edit.h>

void    VUUpdatePad(VU vu)
{
    EDOPTS *opts = (EDOPTS *)vu->vuoptions;

    if(!opts->ed_dirty) {
        opts->ed_dirty = TRUE;
        VUPostRedraw(VU_PAINT, vu, NULL);
    }
}

DEFPORT(VUTextPad)
{
    STRING  title=vu->vutitle;
    RECT    rc;
    int     top=0;
    int len;
    int width;
    int row,col,c;
    CELL    cell;
    PAL pal =   vu->vupal;
    EDOPTS  *opts = (EDOPTS *)vu->vuoptions;
    EDLINE  *ln   = (EDLINE *)vu->vupaint;
    int     lineno;
    unsigned char   nbr[9];
    unsigned char   *p;
    int     n, pow;

    switch(event) {
    case VU_CREATE:
        opts->ed_width = cols;
        opts->ed_left = 0;
        opts->ed_lineno = 0;
        opts->ed_scroll = FALSE;
        opts->ed_wrap = FALSE;
        opts->ed_dirty = TRUE;
        opts->ed_update = FALSE;
        opts->ed_number = 0;
        break;
    case VU_PAINT:
        opts->ed_dirty = FALSE;
        memcpy(&rc,&vu->vudrawn,sizeof(rc));
        len=strlen((char *)title);
        if(VUGetAttr(vu,HASMENUBAR)) {
            c=(rc.rt - rc.lt +1 - len) / 2;
            row=rc.tp;
            col=rc.lt;
            if(!len) {
                c = 0;
                width = rc.rt - rc.lt + 1;
                if(VUGetAttr(vu, HASBORDER)) {
                    ++col;
                    width -= 2;
                }
                pal = (vu->vupal + 6);
                if(speed > 12000 && vu != front && !VUGetAttr(vu, ACTIVE))
                    ++pal;
                while(c++ < width) {
                    if((c % 4))
                        cell = (CELL)(pal << 8 | '.');
                    else
                        cell = (CELL)(pal << 8 | '|');
                    VUDrawCell(cell, row, col++);
                }
            }
            else {
                cell = (vu->vupal + 4) << 8 | 0x20;
                if(!inverse_mode && !color)
                    cell |= '=';
                while(c--)
                    VUDrawCell(cell, row, col++);
                VUDrawString(title, row, col, len, vu->vupal+4);
                col += len;
                while(col <= rc.rt)
                    VUDrawCell(cell, row, col++);
            }
            ++top;
        }
        if(VUGetAttr(vu, HASBORDER)) {
            pal=(PAL)(vu->vupal + 6);
            if(speed > 12000 && vu != front && !VUGetAttr(vu, ACTIVE))
                ++pal;
            VUDrawFrame(vu, pal);
            if(VUGetAttr(vu,HASTITLE))
                VUDrawString(title,rc.tp,rc.lt + 2,len,pal);
            ++top;
            --rc.bt;
            ++rc.lt;
            --rc.rt;
        }
        if(top)
            ++rc.tp;
        width = rc.rt - rc.lt + 1;
        row = rc.tp;
        lineno = opts->ed_lineno;
        while((row <= rc.bt) && ln) {
            len = strlen(ln->ed_data);
            cell = (CELL)((ln->ed_mask & ED_PALMASK) << 8);
            if(opts->ed_number) {
                len = opts->ed_number;
                pow = 1;
                while(--len)
                    pow *= 10;
                len = opts->ed_number - 1;
                p = nbr;
                n = lineno;
                while(--len) {
                    if(n > pow) {
                        *(p++) = (char)(lineno / pow) + '0';
                        n -= (n / pow) * pow;
                    }
                    else
                        *(p++) = 0;
                    pow /= 10;
                }
                *(p++) = ' ';
                *p = 0;
/*
                strcpy(fmt, "%0xd ");
                fmt[2] = (char)('0' + opts->ed_number - 1);
                sprintf(nbr, fmt, lineno);
*/
                VUDrawString(nbr, row, rc.lt, opts->ed_number, vu->vupal);
            }
            col = rc.lt + opts->ed_number;
            len = strlen(ln->ed_data);
            c = opts->ed_left;

            while(col <= rc.rt) {
                if(c < len)
                    VUDrawCell(cell | (CELL)ln->ed_data[c++], row, col++);
                else
                    VUDrawCell(cell | 0x20, row, col++);
            }
            ln = ln->ed_next;
            ++row;
            ++lineno;
        }
        if(row <= rc.bt) {
            rc.tp = row;
            if(!p2)
                VUImageFill(&rc,(vu->vupal << 8) | 0x20);
            else
                VUImageFill(&rc,p2);
        }
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUWindowPort);
}
