/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Misc. time support routines.  Often localized by national group.
 * This function contains those functions common to different national groups
 * which can if necessary be linked out by other localization routines.
 */

#include <vu-config.h>
#include <ctype.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>

#define vu_national vu_national_time

CONST   char    timesep[] = {':', ':', '.', ' '};
CONST   char    clk24[] =   { 0,   1,   1,   1};

DEFCONTROL(VUEditTimeControl)
{
    switch(event) {
    case VU_CREATE:
        if(!control->ctext[0])
            strcpy((char *)control->ctext,"??:??");
        control->ctext[2] = timesep[vu_national];
        break;
    case VU_KEYBOARD:
        if(p1 < 32 || p1 > 126)
            break;
        if(!isdigit(p1) && p1 != ' ') {
            if(isalpha(p1))
                return TRUE;
            else
                return FALSE;
        }
        switch(control->cstate)
        {
        case 0:
            if(p1 > '2')
                return TRUE;
            break;
        case 1:
            if(control->ctext[0] == '2')
                if(p1 > '3')
                    return TRUE;
            break;
        case 3:
            if(p1 > '6')
                return TRUE;
        case 4:
            break;
        }
        p2=FALSE;       /* disable insert mode operation */
        break;
    default:
        break;
    }
    return USECONTROL(VUEditMaskControl);
}

struct  tm* VUScanTime(struct tm *mask, char *buf)
{
    mask->tm_hour=atoi(buf);
    mask->tm_min=atoi(buf+3);
    return mask;
}

char* VUCvtTime(struct tm *now)
{
    static  char    time[7];
    int     hour=now->tm_hour;
    int     min=now->tm_min;
    char        apm = 'a';

    strcpy(time,"00:00");
    time[2] = timesep[vu_national];

    if(!clk24[vu_national]) {
        if(hour > 11) {
            hour -= 12;
            apm = 'p';
        }

        if(!hour)
            hour=12;

        if(hour > 9) {
            time[0]='1';
            hour -= 10;
        }
        time[1]=(char)(hour + '0');
    }
    else {
        time[0] = (char)('0' + (hour / 10));
        time[1] = (char)('0' + (hour % 10));
    }
    time[3]=(char)('0' + (min / 10));
    time[4]=(char)('0' + (min % 10));
    if(!clk24[vu_national])
        time[5]=apm;

    return  time;
}

