/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>
#include <worldvu/edit.h>

const   DEFWINDOW(VUViewFile)
    WIN_CLASS(VUViewFilePad, (STRING)"File Viewer")
    WIN_ATTR(18, 66, HASBORDER | HASSHADOW | HASMENUBAR, 24,
        (STRING)"Press <ESCAPE> to exit")
#ifdef  BUILD_LOCALIZED
    REPLACE_ITALIAN((STRING)"Visualizzatore documento",(STRING)"premi <ESCAPE> per uscire")
    REPLACE_ROMANIAN((STRING)"Citire fisier", (STRING)"Apasati <ESCAPE> pentru abandon")
    REPLACE_ROMANIAN((STRING)"Prohlizec souboru", (STRING)"Pro ukonceni stisknete <ESC>")
#endif
ENDDEF

#define MSG_EOT msgs[replacement_language * 2]
#define MSG_MISSING msgs[replacement_language * 2 + 1]

CONST   char    *msgs[] =
{
    "<EOT>",
    "<file missing>",

#ifdef  BUILD_LOCALIZED
    "<FINE>",
    "<archivio mancante>",

    "<FIN>",
    "<fisier inexistent>",

    "<Konec>",
    "<soubor neexistuje>",

#endif
    NULL
};

DEFPORT(VUViewFilePad)
{
    FILE    *fp;
    char    lbuf[256];
    char    *p;
    int     width = vu->vudrawn.rt - vu->vudrawn.lt + 1;

    switch(event) {
    case VU_KEYBOARD:
        if(p1 == 26 || p1 == 4)
            p1 = KEY_EXIT;
        if(p1 == ' ')
            p1 = KEY_PAGEDOWN;
        if(p1 == KEY_ENTER)
            p1 = KEY_DOWN;
        break;

    case VU_CREATE:

        if(!USEPORT(VUViewPad))
            return FALSE;

        fp = fopen(vu->vupaint, "r");
        if(!fp) {
            VUAddEditText(vu, 26, (char *)MSG_MISSING);
            break;
        }
        for(;;) {
            fgets(lbuf, width, fp);
            if(feof(fp))
                break;

            p = VUTrimString(lbuf, "\n\r", "\n\t\r");
            if(!VUAddEditText(vu, 0, (char *)VUTextString((STRING)VUTrimString(p, "\r", "\r"))))
                break;
        }
        fclose(fp);
        VUAddEditText(vu, 26, "<EOT>");
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUViewPad);
}

