/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/keys.h>
#include <worldvu/edit.h>

#ifndef MAXEDIT
#define MAXEDIT 60000
#endif

DEFPORT(VUViewPad)
{
    EDOPTS  *opts = (EDOPTS *)vu->vuoptions;
    EDCTRL  *ctrl = (EDCTRL *)vu->vuimage;
    EDLINE  *ln;
    int lines;
    int left = 0;
    long    max;
    int width = 0;

    if(VUGetAttr(vu, HASBORDER))
        ++left;

    switch(event) {
    case VU_KEYBOARD:
        if(vu != front)
            break;

        lines = vu->vudrawn.bt - vu->vudrawn.tp + 1;
        if(VUGetAttr(vu, HASBORDER))
            lines -=2;
        else if(VUGetAttr(vu, HASMENUBAR))
            lines -=1;

        ln = ctrl->ed_current;

        switch(p1)
        {
        case 2:
            if(opts->ed_left) {
                opts->ed_left = 0;
                VUUpdatePad(vu);
            }
            return TRUE;
        case 't':
        case 'T':
        case KEY_HOME:
            ctrl->ed_current = ctrl->ed_first;
            opts->ed_lineno = ctrl->ed_clineno = 1;
            vu->vupaint = (char *)ctrl->ed_first;
            VUUpdatePad(vu);
            return TRUE;
        case 'e':
        case 'E':
        case KEY_END:
            ctrl->ed_current = ctrl->ed_last;
            vu->vupaint = (char *)ctrl->ed_last;
            opts->ed_lineno = ctrl->ed_clineno = ctrl->ed_llineno;
            VUScrollPad(vu, - (--lines), FALSE);
            return TRUE;
        case 'u':
        case 'U':
        case KEY_UP:
            lines = 8;
        case 'p':
        case 'P':
        case 0x10:
        case KEY_PAGEUP:
            VUScrollPad(vu, - (--lines), TRUE);
            return TRUE;
        case 'd':
        case 'D':
        case KEY_DOWN:
            lines = 8;
        case 'n':
        case 'N':
        case 0x0e:
        case KEY_PAGEDOWN:
            VUScrollPad(vu, --lines, TRUE);
            return TRUE;
        case 'r':
        case 'R':
        case KEY_RIGHT:
            opts->ed_left += 8;
            VUUpdatePad(vu);
            return TRUE;
        case 'l':
        case 'L':
        case KEY_LEFT:
            if(opts->ed_left)
                opts->ed_left -= 8;
            if(opts->ed_left < 0)
                opts->ed_left = 0;
            VUUpdatePad(vu);
            return TRUE;
        }
        break;
    case VU_CREATE:
        width = p2 / 2;
        p2 &= 0x01;

        if(!USEPORT(VUTextPad))
            return FALSE;

        max = MAXEDIT;

        if(p1 > TRUE) {
            if((unsigned)p1 < MAXEDIT)
                max = p1;
            p1 = TRUE;
        }
        if(width > TRUE)
            opts->ed_width = width;

        vu->vuimage = (PTR)malloc(sizeof(EDCTRL) + (int)(max));
        ctrl = (EDCTRL *)vu->vuimage;
        ln = (EDLINE *)((char *)vu->vuimage + sizeof(EDCTRL));
        if(!ctrl)
            return FALSE;
        lines = (int)(max / (long)(opts->ed_width + 1 + sizeof(EDLINE)));
        ctrl->ed_free = ln;
        ctrl->ed_current = NULL;
        ctrl->ed_first = NULL;
        ctrl->ed_last = NULL;
        ctrl->ed_clineno = 0;
        ctrl->ed_llineno = 0;
        ctrl->ed_statline = (char *)vu->vustatline;
        lines -= 2;
        while(--lines) {
            ln->ed_mask = 0;
            ln->ed_next = (EDLINE *)((char *)ln + opts->ed_width + 1 + sizeof(EDLINE));
            ln = ln->ed_next;
        }
        ln->ed_next = NULL;
        vu->vucurcol = 0;
        ctrl->ed_wrap = (char *)ln + opts->ed_width + 1 + sizeof(EDLINE);
        return TRUE;
    default:
        break;
    }
    return USEPORT(VUTextPad);
}
