/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
    Base window control management.  Lowest level of object inheritence.

    0.5     David Sugar     Feb 26 92

    Button events will no longer cause shutdown of object if p2 is TRUE.
*/

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/multi.h>
#include <worldvu/keys.h>
#include <unistd.h>

static  DEFPORT(VUAllControls)
{
    register    CITEM   *control=vu->vucontrols;

    while(control->cproc) {
        if(!(*(control->cproc))(vu,control,event,p1,p2))
            return FALSE;
        ++control;
    }
    return TRUE;
}

DEFPORT(VUWindowPort)
{
    CITEM   *control=vu->vucontrol;
    STRING  title;
    CITEM       *base=control;
    int rtn;
    RECT    rc;
    int     top=0;
    int len;
    int row,col,c;
    CELL    cell;
    PAL pal =   vu->vupal;
    char    *env;

    memcpy(&rc,&vu->vudrawn,sizeof(rc));

    switch(event) {
    case VU_REGION:
        if(!vu->vunotify)
            return FALSE;

        return FALSE;
    case VU_NOTIFY:             /* default behavior is for children to die */
        return TRUE;
    case VU_PAINT:
        title = VUTextString(vu->vutitle);
        len=strlen((char *)title);
        if(VUGetAttr(vu,HASMENUBAR)) {
            c=(rc.rt - rc.lt +1 - len) / 2;
            row=rc.tp;
            col=rc.lt;
            cell = (vu->vupal + 4) << 8 | 0x20;
            if(!inverse_mode && !color)
                cell |= '=';
            while(c--)
                VUDrawCell(cell,row,col++);
            VUDrawString(title,row,col,len,vu->vupal+4);
            col+=len;
            while(col <= rc.rt)
                VUDrawCell(cell,row,col++);
            ++top;
        }
        if(VUGetAttr(vu,HASBORDER)) {
            pal=(PAL)(vu->vupal + 6);
            if(speed > 12000 && vu != front && !VUGetAttr(vu,ACTIVE))
                ++pal;
            VUDrawFrame(vu,pal);
            if(VUGetAttr(vu,HASTITLE))
                VUDrawString(title,rc.tp,rc.lt + 2,len,pal);
            ++top;
            --rc.bt;
            ++rc.lt;
            --rc.rt;
        }
        if(top)
            ++rc.tp;
        if(!p2)
            VUImageFill(&rc,(vu->vupal << 8) | 0x20);
        else
            VUImageFill(&rc,p2);
        p2=FALSE;
        return USEPORT(VUAllControls);

    case VU_FRONT:
        if(vu->vunotify)
            VUUpdateRegions();

        if(VUGetAttr(vu,ACTIVE))
            return TRUE;
        if(VUGetAttr(vu,HASBORDER) && speed > 12000 ) {
            pal=(PAL)(vu->vupal + 6);
            if(!p1)
                ++pal;
            VUDrawFrame(vu,pal);
        }
        return TRUE;
    case VU_CREATE:
        vu->vucontrol=vu->vucontrols;
        if(!vu->vuresid)
            if(vu->vuparent)
                vu->vuresid = vu->vuparent->vuresid;
        rtn= USEPORT(VUAllControls);
        if(rtn && !p2)
            VUToFront(vu);
        return rtn;
    case VU_BUTTON:                         /* destruct on button event */
        if(!p2)
            VUPostEvent(vu,VU_DESTROY,p1,FALSE);
        return TRUE;
    case VU_DESTROY:
        if(vu->vunotify)
            VUUpdateRegions();

        if(!p2) {
            if(vu->vuparent)
                VUPostEvent(vu->vuparent,VU_NOTIFY,vu->vuresid,p1);
        }
        VUAllControls(vu,event,p1,p2);

        VUDestroy(vu);
        if(VUGetAttr(vu,DESKTOP))
            VUPostEvent(desktop,VU_SHUTDOWN,0,0);
        return TRUE;
    case VU_IDLE:
        return FALSE;
    case VU_KEYBOARD:

        title = vu->vutitle;
        if(vu != front && VUGetAttr(vu,MOVEFRONT)) {
            while(*title && *title != '~')
            ++title;

            if(*title == '~')
                if(VUKeyMatch(*(++title),p1)) {
                    VUToFront(vu);
                    return TRUE;
                }

            return FALSE;
        }

        if(p1 == KEY_EXIT) {
            if(VUGetAttr(vu,FAILURE))
                VUPostEvent(desktop,VU_SHUTDOWN,0,0);
            else if(VUGetAttr(vu,DESKTOP)) {
                if(shutdown_mode)
                    VUPostEvent(desktop,VU_SHUTDOWN,0,0);
                else {
                    env = getenv("VU_SHELL");
                    if(env)
                        if(atoi(env) == getppid()) {
                            VUPostEvent(desktop, VU_SHUTDOWN, 0, 0);
                            return TRUE;
                        }
                    VUPostEvent(desktop,VU_KEYBOARD,3,0);
                }
            }
            else if(!VUGetAttr(vu,LOCKED))
                VUPostEvent(vu,VU_DESTROY,0,FALSE);
            return TRUE;
        }

        if(!control->cproc)
            return FALSE;

        if((*(control->cproc))(vu,control,event,p1,p2))
            return TRUE;

        if(p1 == KEY_UP || p1 == KEY_LEFT || p1 == KEY_PAGEUP) {
            for(;;) {
                if(control == vu->vucontrols)
                    return FALSE;
                --control;
                if((*(control->cproc))(vu,control,event,p1,p2))
                    return TRUE;
            }
        }
        else {
            ++control;
            while(control != base) {
                if(!control->cproc) {
                    if(p1 != KEY_DOWN && p1 != KEY_RIGHT && p1 != KEY_PAGEDOWN)
                        control=vu->vucontrols;
                    else
                        return FALSE;
                }
                else {
                    if((*(control->cproc))(vu,control,event,p1,p2))
                        return TRUE;
                    ++control;
                }
            }
        }
        return FALSE;
    default:
        break;
    }
    return FALSE;
}
