/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/term.h>
#include <worldvu/keys.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

FILE    *fp;

#ifdef  QNX
#define RWMODE  "rw"
#define WOMODE  "w"
#endif

#ifndef RWMODE
#define RWMODE  "r+b"
#endif

#ifndef WOMODE
#define WOMODE  "wb"
#endif

/* #define	TITLE	"Color Set Editor" */
#define	TITLE NULL

enum
{
	ID_FCOLOR=VU_USER,
	ID_BCOLOR,
	ID_SHADOW,
	ID_INVERSE,
	ID_BRIGHT,
	ID_UNDERLINE,
	ID_BLINK
};

typedef	struct
{
	short	pal_entry;
	short	pal_fcolor;
	short	pal_bcolor;
	short	pal_inverse;
	short	pal_bright;
	short	pal_underline;
	short	pal_blink;
	char	pal_shadow[3];
}	PALINFO;

FORWARD PORT(EntPort);
FORWARD PORT(PalPort);

static	PALINFO		entry;

CONST	char	*fcolors[]= 
{"Dark Black", "Dark Blue", "Dark Green", "Dark Cyan", "Dark Red", "Dark Magenta", "Dark Brown", "Dark White",
 "Light Grey", "Light Blue","Light Green","Light Cyan","Light Red","Light Magenta","Light Brown","Light White",NULL};

CONST	char	*bcolors[]=
{"Black","Blue","Green","Cyan","Red","Magenta","Brown","White",NULL};

CONST	char	*monopts[]=
{"Never","Hinting","Always",NULL};

CONST	DEFWINDOW(PalTable)
	WIN_CLASS(PalPort,"Color Palette")
	WIN_ATTR(17, 33, HASSHADOW | HASBORDER | HASMENUBAR | DESKTOP, 16, "Press <SPACE> to alter palette entry")

	TEXT_FIELD("Application",	1, 18, 1, 16, 1)
	TEXT_FIELD("Controls",      3, 18, 1, 16, 1)
	TEXT_FIELD("Form Window",	5, 18, 1, 16, 1)
	TEXT_FIELD("Dialog Window", 7, 18, 1, 16, 1)
	TEXT_FIELD("Alert Window",	9, 18, 1, 16, 1)
	TEXT_FIELD("Notify Window",11, 18, 1, 16, 1)
	TEXT_FIELD("Help Window",  13, 18, 1, 16, 1)
	TEXT_FIELD("Card Window",  15, 18, 1, 16, 1)
ENDDEF

CONST	DEFWINDOW(EntTable)
	WIN_CLASS(EntPort,"Edit Palette Entry")
	WIN_ATTR(15, 60, HASSHADOW | HASBORDER | HASMENUBAR, 24, "Press <ESCAPE> to abort")

	BOX_CONTROL("Colors",				2, 2, 5, 28, 0)
	TEXT_CONTROL("Foreground: ",		3, 3, 1, 12, ID_FCOLOR)
	LIST_CONTROL(&entry.pal_fcolor,		3,16, 5, 12, ID_FCOLOR, fcolors)
	TEXT_CONTROL("Background: ",        5, 3, 1, 12, ID_BCOLOR)
	LIST_CONTROL(&entry.pal_bcolor,     5,16, 5, 12, ID_BCOLOR, bcolors)
	
	TEXT_CONTROL("Shadow entry:",       8, 3, 1, 13, ID_SHADOW)
	EDITNUM_CONTROL(entry.pal_shadow,  8,16, 2, ID_SHADOW,"Must specify shadow")

	BOX_CONTROL("Monochrome",			2,32, 9, 24, 0)
	TEXT_CONTROL("Inverse:",			3,33, 1, 10, ID_INVERSE)
	LIST_CONTROL(&entry.pal_inverse,    3,44, 3,  8, ID_INVERSE,monopts)
	TEXT_CONTROL("Bright:",             5,33, 1, 10, ID_BRIGHT)
	LIST_CONTROL(&entry.pal_bright,     5,44, 3,  8, ID_BRIGHT,monopts)
	TEXT_CONTROL("Underline:",          7,33, 1, 10, ID_UNDERLINE)
	LIST_CONTROL(&entry.pal_underline,  7,44, 3,  8, ID_UNDERLINE,monopts)
	TEXT_CONTROL("Blinking:",           9,33, 1, 10, ID_BLINK)
	LIST_CONTROL(&entry.pal_blink,      9,44, 3,  8, ID_BLINK,monopts)

	BUTTON_CONTROL(" ~Accept ",         12, 10, 8, VU_ACCEPT)
	BUTTON_CONTROL(" ~Cancel ",         12, 42, 8, VU_CANCEL)
ENDDEF
	
DEFPORT(EntPort)
{
	static  PALDEF  oldattr;
	static  ATTR    new;

	switch(event)
	{
	case VU_BUTTON:
		if(p1 == VU_ACCEPT)
		{
			oldattr.palattr=new;
			oldattr.palshadow=(PAL)atoi(entry.pal_shadow);
		}
		break;
	case VU_CREATE:
		memcpy(&oldattr,&palette[entry.pal_entry],sizeof(PALDEF));
		break;
	case VU_NOTIFY:
	case VU_FRONT:
		new=entry.pal_fcolor | (entry.pal_bcolor << 4);
		switch(entry.pal_inverse)
		{
		case 2:
			new |= INVERSE;
			break;
		case 1:
			new |= SUBINVERSE;
			break;
		}
		switch(entry.pal_bright)
		{
		case 2:
			new |= BRIGHT;
			break;
		case 1:
			new |= SUBBRIGHT;
			break;
		}
		switch(entry.pal_underline)
		{
		case 2:
			new |= BRIGHT;
			break;
		case 1:
			new |= SUBBRIGHT;
			break;
		}
		if(entry.pal_blink)
			new |= BLINK;
		VUSetPalette(entry.pal_entry, new, atoi(entry.pal_shadow));
		break;
	case VU_DESTROY:
		VUSetPalette(entry.pal_entry,oldattr.palattr,oldattr.palshadow);
		break;
	}
	return USEPORT(VUWindowPort);
}

DEFPORT(PalPort)
{
	int rtn;
	int pos;
	static  unsigned r=0,c=0;
	static  unsigned r1=0,c1=0;
	ATTR    attr;

	switch(event)
	{
	case VU_CREATE:
		vu->vucurmode = 1;
		vu->vucurrow = 1;
		vu->vucurcol = 1;
		break;
	case VU_PAINT_SELECT:
		VUDrawCell(BOX_U | BOX_D | GRAPHIC_CELL | (vu->vupal << 8), vu->vudrawn.tp + 1 + 2 * r1, vu->vudrawn.lt + 2 * c1);
		VUDrawCell(BOX_U | BOX_D | GRAPHIC_CELL | (vu->vupal << 8), vu->vudrawn.tp + 1 + 2 * r1, vu->vudrawn.lt +2 + 2 * c1);
		r1 = r;
		c1 = c;
		VUDrawCell('['| ((vu->vupal + 6) << 8), vu->vudrawn.tp + 1 + 2 * r, vu->vudrawn.lt + 2 * c);
		VUDrawCell(']'| ((vu->vupal + 6) << 8), vu->vudrawn.tp + 1 + 2 * r, vu->vudrawn.lt +2 + 2 * c);
		vu->vucurrow = 1 + 2 * r;
		vu->vucurcol = 1 + 2 * c; 
		fixcursor();
		return TRUE;
	case VU_FRONT:
		if(!p1)
			return TRUE;
	case VU_PAINT:
		rtn=USEPORT(VUWindowPort);
		VUDrawFrame(vu,vu->vupal);                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
		for(pos=vu->vudrawn.tp + 2; pos < vu->vudrawn.bt; pos += 2)
			VUDrawHLine(pos,vu->vudrawn.lt, vu->vudrawn.rt - 16 - vu->vudrawn.lt + 1);
		for(pos=vu->vudrawn.lt + 2; pos < vu->vudrawn.rt - 15; pos += 2)
			VUDrawVLine(vu->vudrawn.tp + 1,pos, vu->vudrawn.bt - vu->vudrawn.tp);
		for(r1 = 0; r1 < 8; ++r1)
			for(c1 = 0; c1 < 8; ++c1)
				VUDrawCell('*' | ((r1 * 8 + c1) << 8), vu->vudrawn.tp + 1 + 2 * r1, vu->vudrawn.lt + 1 + 2 * c1);
		VUDrawCell('['| ((vu->vupal + 6) << 8), vu->vudrawn.tp + 1 + 2 * r, vu->vudrawn.lt + 2 * c);
		VUDrawCell(']'| ((vu->vupal + 6) << 8), vu->vudrawn.tp + 1 + 2 * r, vu->vudrawn.lt +2 + 2 * c);
		r1 = r;
		c1 = c;
		return rtn;
	case VU_KEYBOARD:
		switch(p1)
		{
		case ' ':
		case KEY_ENTER:
			pos = 8 * r + c;
			entry.pal_entry=pos;
			sprintf(entry.pal_shadow,"%d",palette[pos].palshadow);
			attr = palette[pos].palattr;
			entry.pal_bcolor=(attr & 0x70) >> 4;
			entry.pal_fcolor=attr & 0x0f;
			if(attr & INVERSE)
				entry.pal_inverse=2;
			else if(attr & SUBINVERSE)
				entry.pal_inverse=1;
			else
				entry.pal_inverse=0;
			if(attr & BRIGHT)
				entry.pal_bright=2;
			else if(attr & SUBBRIGHT)
				entry.pal_bright=1;
			else
				entry.pal_bright=0;
			if(attr & UNDERLINE)
				entry.pal_underline=2;
			else if(attr & SUBUNDERLINE)
				entry.pal_underline=1;
			else
				entry.pal_underline=0;
			if(attr & BLINK)
				entry.pal_blink=2;
			else
				entry.pal_blink=0;
			VUCreateQ(EntTable,NULL,-1,18,NULL);
			return TRUE;
		case KEY_DOWN:
			++r;
			r %= 8;
			VUPostRedraw(VU_PAINT_SELECT,vu,NULL);
			return TRUE;
		case KEY_UP:
			--r;
			r %= 8;
			VUPostRedraw(VU_PAINT_SELECT,vu,NULL);
			return TRUE;
		case KEY_LEFT:
			--c;
			c %= 8;
			VUPostRedraw(VU_PAINT_SELECT,vu,NULL);
			return TRUE;
		case KEY_RIGHT:
			++c;
			c %= 8;
			VUPostRedraw(VU_PAINT_SELECT,vu,NULL);
			return TRUE;
		}
	}
	return USEPORT(VUWindowPort);
}

PROGRAM
{
	if(argc != 2)
/*	{
		fputs("use: vucolors set_name\n",stdout);
		exit(-1);
	}
*/
		argv[1] = "test";

	if(NULL != (fp=fopen(argv[1],RWMODE)))
	{
		fread(palette,sizeof(PALDEF),64,fp);
		fseek(fp,0l,SEEK_SET);
	}
	else if(NULL == (fp=fopen(argv[1],WOMODE)))
	{
		fputs("vucolors: cannot access set\n",stdout);
		exit(-1);
	}
	
	VUStartup();

	VUCreateQ(VUDesktop,TITLE,-1,-1,NULL);
	VUCreateQ(PalTable,NULL, 1, 1, NULL);

	while(VUDispatch())
		;

	VUShutdown();
	fwrite(palette,sizeof(PALDEF),64,fp);
	fclose(fp);
	exit(0);
}
	
	
