/* Copyright (C) 1992-2014 David Sugar, Tycho Softworks.
 *
 * This file is part of worldvu.
 *
 * worldvu is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * worldvuvu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with worldvu.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vu-config.h>
#include <worldvu/vu.h>
#include <worldvu/term.h>
#include <stdio.h>
#include <string.h>

static size_t width = 0;

static void output(const char *text)
{
    size_t len = strlen(text);
    ++len;
    if(width + len > 76) {
        width = len;
        printf("\n %s", text);
    }
    else {
        width += len;
        printf(" %s", text);
    }
}

static void bputs(const char *prefix, bool mode)
{
    char buf[32];
    if(mode)
        snprintf(buf, sizeof(buf), "%s=true", prefix);
    else
        snprintf(buf, sizeof(buf), "%s=false", prefix);
    output(buf);
}

static void tputs(const char *prefix, STRING str, unsigned max)
{
    char buf[64];
    unsigned pos;

    if(!*str)
        return;

    snprintf(buf, sizeof(buf), "%s=", prefix);
    while(*str || max) {
        pos = strlen(buf);
        if(*str == '^' || *str == '#')
            snprintf(buf + pos, sizeof(buf) - pos, "\\%c", *str);
        else if(*str < 32)
            snprintf(buf + pos, sizeof(buf) - pos, "^%c", *str + 64);
        else if(*str > 126 || *str == 32 || *str == 0)
            snprintf(buf + pos, sizeof(buf) - pos, "#%02x", *str);
        else
            snprintf(buf + pos, sizeof(buf) - pos, "%c", *str);
        ++str;
        if(max) {
            if(!(--max))
                break;
        }
    }
    output(buf);
}

static void show(TERMDEF *db)
{
    printf("%s: \"%s\" %sx%s\n", db->term_id, db->term_name, db->term_cols, db->term_rows);
    tputs("startup", (STRING)db->term_startup, 0);
    tputs("shutdown", (STRING)db->term_startup, 0);
    tputs("beep", (STRING)db->term_beep, 0);
    tputs("cls", (STRING)db->term_cls, 0);
    tputs("eol", (STRING)db->term_eol, 0);
    tputs("eop", (STRING)db->term_eop, 0);
    tputs("hide", (STRING)db->term_cursor_hide, 0);
    tputs("show", (STRING)db->term_cursor_show, 0);
    tputs("big", (STRING)db->term_cursor_big, 0);
    tputs("go-left", (STRING)db->term_left, 0);
    tputs("go-right", (STRING)db->term_right, 0);
    tputs("go-up", (STRING)db->term_up, 0);
    tputs("go-down", (STRING)db->term_down, 0);
    tputs("go-begin", (STRING)db->term_cursor_beg, 0);
    tputs("go-end", (STRING)db->term_cursor_end, 0);
    tputs("go-sep", (STRING)db->term_cursor_sep, 0);
    tputs("go-vt", (STRING)db->term_cursor_vt, 0);
    tputs("go-offset", (STRING)db->term_cursor_offset, 0);
    bputs("go-rc", db->term_cursor_order);
    bputs("go-dec", db->term_cursor_decimal);
    tputs("at-begin", (STRING)db->term_attr_beg, 0);
    tputs("at-end", (STRING)db->term_attr_end, 0);
    tputs("at-sep", (STRING)db->term_attr_sep, 0);
    bputs("at-merge", db->term_attr_merge);
    tputs("at-clear", (STRING)db->term_attr_clear, 0);
    tputs("inv-on", (STRING)db->term_attr_inverse_on, 0);
    tputs("inv-off", (STRING)db->term_attr_inverse_off, 0);
    tputs("bold-on", (STRING)db->term_attr_bright_on, 0);
    tputs("bold-off", (STRING)db->term_attr_bright_off, 0);
    tputs("ul-on", (STRING)db->term_attr_underline_on, 0);
    tputs("ul-off", (STRING)db->term_attr_underline_off, 0);
    tputs("gr-on", (STRING)db->term_attr_graphics_on, 0);
    tputs("gr-off", (STRING)db->term_attr_graphics_off, 0);
    tputs("blink-on", (STRING)db->term_attr_blink_on, 0);
    tputs("blink-off", (STRING)db->term_attr_blink_off, 0);
    printf("\n");
    width = 0;
}

static void dump(const char *path)
{
    TERMDEF *db;

    if(!OpenVUTermlib(path)) {
        fprintf(stderr, "*** vutermlib: %s: terminal db missing\n", path);
        exit(2);
    }

    db = FirstVUTermlib();
    while(db != NULL) {
        show(db);
        db = NextVUTermlib();
    }
    exit(0);
}

int main(int argc, char **argv)
{
    char *opt;

    if(argc < 2)
        dump(PREFIX_CONFDIR "/term.db");

    opt = argv[1];
    if(*opt == '-')
        ++opt;
    if(!strcmp(opt, "d") || !strcmp(opt, "dump")) {
        if(!argv[2]) {
            fprintf(stderr, "*** vutermlib: %s: missing file argument\n", argv[1]);
            exit(3);
        }
        dump(argv[2]);
    }

    fprintf(stderr, "*** vutermlib: unknown option\n");
    exit(-1);
}
